# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Instance(pulumi.CustomResource):
    allocate_public_ip: pulumi.Output[bool]
    """
    It has been deprecated from version "1.7.0". Setting "internet_max_bandwidth_out" larger than 0 can allocate a public ip address for an instance.
    """
    auto_release_time: pulumi.Output[str]
    """
    The automatic release time of the `PostPaid` instance. 
    The time follows the ISO 8601 standard and is in UTC time. Format: yyyy-MM-ddTHH:mm:ssZ. It must be at least half an hour later than the current time and less than 3 years since the current time.
    Set it to null can cancel automatic release attribute and the ECS instance will not be released automatically.
    """
    auto_renew_period: pulumi.Output[float]
    """
    Auto renewal period of an instance, in the unit of month. It is valid when `instance_charge_type` is `PrePaid`. Default to 1. Valid value:
    - [1, 2, 3, 6, 12] when `period_unit` in "Month"
    - [1, 2, 3] when `period_unit` in "Week"
    """
    availability_zone: pulumi.Output[str]
    """
    The Zone to start the instance in. It is ignored and will be computed when set `vswitch_id`.
    """
    credit_specification: pulumi.Output[str]
    """
    Performance mode of the t5 burstable instance. Valid values: 'Standard', 'Unlimited'.
    """
    data_disks: pulumi.Output[list]
    """
    The list of data disks created with instance.
    
      * `category` (`str`) - The category of the disk:
        - `cloud`: The general cloud disk.
        - `cloud_efficiency`: The efficiency cloud disk.
        - `cloud_ssd`: The SSD cloud disk.
        - `cloud_essd`: The ESSD cloud disk.
        - `ephemeral_ssd`: The local SSD disk.
        Default to `cloud_efficiency`.
        * `encrypted` -(Optional, Bool, ForceNew) Encrypted the data in this disk.
      * `delete_with_instance` (`bool`) - Delete this data disk when the instance is destroyed. It only works on cloud, cloud_efficiency, cloud_essd, cloud_ssd disk. If the category of this data disk was ephemeral_ssd, please don't set this param.
      * `description` (`str`) - The description of the data disk.
      * `encrypted` (`bool`)
      * `name` (`str`) - The name of the data disk.
      * `size` (`float`) - The size of the data disk.
        - cloud：[5, 2000]
        - cloud_efficiency：[20, 32768]
        - cloud_ssd：[20, 32768]
        - cloud_essd：[20, 32768]
        - ephemeral_ssd: [5, 800]
      * `snapshot_id` (`str`) - The snapshot ID used to initialize the data disk. If the size specified by snapshot is greater that the size of the disk, use the size specified by snapshot as the size of the data disk.
    """
    deletion_protection: pulumi.Output[bool]
    """
    Whether enable the deletion protection or not.
    - true: Enable deletion protection.
    - false: Disable deletion protection.
    """
    description: pulumi.Output[str]
    """
    The description of the data disk.
    """
    dry_run: pulumi.Output[bool]
    """
    Specifies whether to send a dry-run request. Default to false. 
    - true: Only a dry-run request is sent and no instance is created. The system checks whether the required parameters are set, and validates the request format, service permissions, and available ECS instances. If the validation fails, the corresponding error code is returned. If the validation succeeds, the `DryRunOperation` error code is returned.
    - false: A request is sent. If the validation succeeds, the instance is created.
    """
    force_delete: pulumi.Output[bool]
    """
    If it is true, the "PrePaid" instance will be change to "PostPaid" and then deleted forcibly.
    However, because of changing instance charge type has CPU core count quota limitation, so strongly recommand that "Don't modify instance charge type frequentlly in one month".
    """
    host_name: pulumi.Output[str]
    """
    Host name of the ECS, which is a string of at least two characters. “hostname” cannot start or end with “.” or “-“. In addition, two or more consecutive “.” or “-“ symbols are not allowed. On Windows, the host name can contain a maximum of 15 characters, which can be a combination of uppercase/lowercase letters, numerals, and “-“. The host name cannot contain dots (“.”) or contain only numeric characters. When it is changed, the instance will reboot to make the change take effect.
    On other OSs such as Linux, the host name can contain a maximum of 30 characters, which can be segments separated by dots (“.”), where each segment can contain uppercase/lowercase letters, numerals, or “_“. When it is changed, the instance will reboot to make the change take effect.
    """
    image_id: pulumi.Output[str]
    """
    The Image to use for the instance. ECS instance's image can be replaced via changing 'image_id'. When it is changed, the instance will reboot to make the change take effect.
    """
    include_data_disks: pulumi.Output[bool]
    """
    Whether to change instance disks charge type when changing instance charge type.
    """
    instance_charge_type: pulumi.Output[str]
    """
    Valid values are `PrePaid`, `PostPaid`, The default is `PostPaid`.
    """
    instance_name: pulumi.Output[str]
    instance_type: pulumi.Output[str]
    """
    The type of instance to start. When it is changed, the instance will reboot to make the change take effect.
    """
    internet_charge_type: pulumi.Output[str]
    """
    Internet charge type of the instance, Valid values are `PayByBandwidth`, `PayByTraffic`. Default is `PayByTraffic`. At present, 'PrePaid' instance cannot change the value to "PayByBandwidth" from "PayByTraffic".
    """
    internet_max_bandwidth_in: pulumi.Output[float]
    """
    Maximum incoming bandwidth from the public network, measured in Mbps (Mega bit per second). Value range: [1, 200]. If this value is not specified, then automatically sets it to 200 Mbps.
    """
    internet_max_bandwidth_out: pulumi.Output[float]
    """
    Maximum outgoing bandwidth to the public network, measured in Mbps (Mega bit per second). Value range:  [0, 100]. Default to 0 Mbps.
    """
    io_optimized: pulumi.Output[str]
    """
    It has been deprecated on instance resource. All the launched alicloud instances will be I/O optimized.
    """
    is_outdated: pulumi.Output[bool]
    """
    Whether to use outdated instance type. Default to false.
    """
    key_name: pulumi.Output[str]
    """
    The name of key pair that can login ECS instance successfully without password. If it is specified, the password would be invalid.
    """
    kms_encrypted_password: pulumi.Output[str]
    """
    An KMS encrypts password used to an instance. If the `password` is filled in, this field will be ignored. When it is changed, the instance will reboot to make the change take effect.
    """
    kms_encryption_context: pulumi.Output[dict]
    """
    An KMS encryption context used to decrypt `kms_encrypted_password` before creating or updating an instance with `kms_encrypted_password`. See [Encryption Context](https://www.alibabacloud.com/help/doc-detail/42975.htm). It is valid when `kms_encrypted_password` is set. When it is changed, the instance will reboot to make the change take effect.
    """
    password: pulumi.Output[str]
    """
    Password to an instance is a string of 8 to 30 characters. It must contain uppercase/lowercase letters and numerals, but cannot contain special symbols. When it is changed, the instance will reboot to make the change take effect.
    """
    period: pulumi.Output[float]
    """
    The duration that you will buy the resource, in month. It is valid when `instance_charge_type` is `PrePaid`. Default to 1. Valid values:
    - [1-9, 12, 24, 36, 48, 60] when `period_unit` in "Month"
    - [1-3] when `period_unit` in "Week"
    """
    period_unit: pulumi.Output[str]
    """
    The duration unit that you will buy the resource. It is valid when `instance_charge_type` is 'PrePaid'. Valid value: ["Week", "Month"]. Default to "Month".
    """
    private_ip: pulumi.Output[str]
    """
    Instance private IP address can be specified when you creating new instance. It is valid when `vswitch_id` is specified. When it is changed, the instance will reboot to make the change take effect.
    """
    public_ip: pulumi.Output[str]
    """
    The instance public ip.
    """
    renewal_status: pulumi.Output[str]
    """
    Whether to renew an ECS instance automatically or not. It is valid when `instance_charge_type` is `PrePaid`. Default to "Normal". Valid values:
    - `AutoRenewal`: Enable auto renewal.
    - `Normal`: Disable auto renewal.
    - `NotRenewal`: No renewal any longer. After you specify this value, Alibaba Cloud stop sending notification of instance expiry, and only gives a brief reminder on the third day before the instance expiry.
    """
    resource_group_id: pulumi.Output[str]
    """
    The Id of resource group which the instance belongs.
    """
    role_name: pulumi.Output[str]
    """
    Instance RAM role name. The name is provided and maintained by RAM. You can use `ram.Role` to create a new one.
    """
    security_enhancement_strategy: pulumi.Output[str]
    """
    The security enhancement strategy.
    - Active: Enable security enhancement strategy, it only works on system images.
    - Deactive: Disable security enhancement strategy, it works on all images.
    """
    security_groups: pulumi.Output[list]
    """
    A list of security group ids to associate with.
    """
    spot_price_limit: pulumi.Output[float]
    """
    The hourly price threshold of a instance, and it takes effect only when parameter 'spot_strategy' is 'SpotWithPriceLimit'. Three decimals is allowed at most.
    """
    spot_strategy: pulumi.Output[str]
    """
    The spot strategy of a Pay-As-You-Go instance, and it takes effect only when parameter `instance_charge_type` is 'PostPaid'. Value range:
    - NoSpot: A regular Pay-As-You-Go instance.
    - SpotWithPriceLimit: A price threshold for a spot instance
    - SpotAsPriceGo: A price that is based on the highest Pay-As-You-Go instance
    """
    status: pulumi.Output[str]
    """
    The instance status.
    """
    subnet_id: pulumi.Output[str]
    system_disk_category: pulumi.Output[str]
    """
    Valid values are `ephemeral_ssd`, `cloud_efficiency`, `cloud_ssd`, `cloud_essd`, `cloud`. `cloud` only is used to some none I/O optimized instance. Default to `cloud_efficiency`.
    """
    system_disk_size: pulumi.Output[float]
    """
    Size of the system disk, measured in GiB. Value range: [20, 500]. The specified value must be equal to or greater than max{20, Imagesize}. Default value: max{40, ImageSize}. ECS instance's system disk can be reset when replacing system disk. When it is changed, the instance will reboot to make the change take effect.
    """
    tags: pulumi.Output[dict]
    """
    A mapping of tags to assign to the resource.
    - Key: It can be up to 64 characters in length. It cannot begin with "aliyun", "acs:", "http://", or "https://". It cannot be a null string.
    - Value: It can be up to 128 characters in length. It cannot begin with "aliyun", "acs:", "http://", or "https://". It can be a null string.
    """
    user_data: pulumi.Output[str]
    """
    User-defined data to customize the startup behaviors of an ECS instance and to pass data into an ECS instance. From version 1.60.0, it can be update in-place. If updated, the instance will reboot to make the change take effect. Note: Not all of changes will take effect and it depends on [cloud-init module type](https://cloudinit.readthedocs.io/en/latest/topics/modules.html).
    """
    volume_tags: pulumi.Output[dict]
    """
    A mapping of tags to assign to the devices created by the instance at launch time.
    - Key: It can be up to 64 characters in length. It cannot begin with "aliyun", "acs:", "http://", or "https://". It cannot be a null string.
    - Value: It can be up to 128 characters in length. It cannot begin with "aliyun", "acs:", "http://", or "https://". It can be a null string.
    """
    vswitch_id: pulumi.Output[str]
    """
    The virtual switch ID to launch in VPC. This parameter must be set unless you can create classic network instances. When it is changed, the instance will reboot to make the change take effect.
    """
    def __init__(__self__, resource_name, opts=None, allocate_public_ip=None, auto_release_time=None, auto_renew_period=None, availability_zone=None, credit_specification=None, data_disks=None, deletion_protection=None, description=None, dry_run=None, force_delete=None, host_name=None, image_id=None, include_data_disks=None, instance_charge_type=None, instance_name=None, instance_type=None, internet_charge_type=None, internet_max_bandwidth_in=None, internet_max_bandwidth_out=None, io_optimized=None, is_outdated=None, key_name=None, kms_encrypted_password=None, kms_encryption_context=None, password=None, period=None, period_unit=None, private_ip=None, renewal_status=None, resource_group_id=None, role_name=None, security_enhancement_strategy=None, security_groups=None, spot_price_limit=None, spot_strategy=None, subnet_id=None, system_disk_category=None, system_disk_size=None, tags=None, user_data=None, volume_tags=None, vswitch_id=None, __props__=None, __name__=None, __opts__=None):
        """
        Create a Instance resource with the given unique name, props, and options.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] allocate_public_ip: It has been deprecated from version "1.7.0". Setting "internet_max_bandwidth_out" larger than 0 can allocate a public ip address for an instance.
        :param pulumi.Input[str] auto_release_time: The automatic release time of the `PostPaid` instance. 
               The time follows the ISO 8601 standard and is in UTC time. Format: yyyy-MM-ddTHH:mm:ssZ. It must be at least half an hour later than the current time and less than 3 years since the current time.
               Set it to null can cancel automatic release attribute and the ECS instance will not be released automatically.
        :param pulumi.Input[float] auto_renew_period: Auto renewal period of an instance, in the unit of month. It is valid when `instance_charge_type` is `PrePaid`. Default to 1. Valid value:
               - [1, 2, 3, 6, 12] when `period_unit` in "Month"
               - [1, 2, 3] when `period_unit` in "Week"
        :param pulumi.Input[str] availability_zone: The Zone to start the instance in. It is ignored and will be computed when set `vswitch_id`.
        :param pulumi.Input[str] credit_specification: Performance mode of the t5 burstable instance. Valid values: 'Standard', 'Unlimited'.
        :param pulumi.Input[list] data_disks: The list of data disks created with instance.
        :param pulumi.Input[bool] deletion_protection: Whether enable the deletion protection or not.
               - true: Enable deletion protection.
               - false: Disable deletion protection.
        :param pulumi.Input[str] description: The description of the data disk.
        :param pulumi.Input[bool] dry_run: Specifies whether to send a dry-run request. Default to false. 
               - true: Only a dry-run request is sent and no instance is created. The system checks whether the required parameters are set, and validates the request format, service permissions, and available ECS instances. If the validation fails, the corresponding error code is returned. If the validation succeeds, the `DryRunOperation` error code is returned.
               - false: A request is sent. If the validation succeeds, the instance is created.
        :param pulumi.Input[bool] force_delete: If it is true, the "PrePaid" instance will be change to "PostPaid" and then deleted forcibly.
               However, because of changing instance charge type has CPU core count quota limitation, so strongly recommand that "Don't modify instance charge type frequentlly in one month".
        :param pulumi.Input[str] host_name: Host name of the ECS, which is a string of at least two characters. “hostname” cannot start or end with “.” or “-“. In addition, two or more consecutive “.” or “-“ symbols are not allowed. On Windows, the host name can contain a maximum of 15 characters, which can be a combination of uppercase/lowercase letters, numerals, and “-“. The host name cannot contain dots (“.”) or contain only numeric characters. When it is changed, the instance will reboot to make the change take effect.
               On other OSs such as Linux, the host name can contain a maximum of 30 characters, which can be segments separated by dots (“.”), where each segment can contain uppercase/lowercase letters, numerals, or “_“. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[str] image_id: The Image to use for the instance. ECS instance's image can be replaced via changing 'image_id'. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[bool] include_data_disks: Whether to change instance disks charge type when changing instance charge type.
        :param pulumi.Input[str] instance_charge_type: Valid values are `PrePaid`, `PostPaid`, The default is `PostPaid`.
        :param pulumi.Input[str] instance_type: The type of instance to start. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[str] internet_charge_type: Internet charge type of the instance, Valid values are `PayByBandwidth`, `PayByTraffic`. Default is `PayByTraffic`. At present, 'PrePaid' instance cannot change the value to "PayByBandwidth" from "PayByTraffic".
        :param pulumi.Input[float] internet_max_bandwidth_in: Maximum incoming bandwidth from the public network, measured in Mbps (Mega bit per second). Value range: [1, 200]. If this value is not specified, then automatically sets it to 200 Mbps.
        :param pulumi.Input[float] internet_max_bandwidth_out: Maximum outgoing bandwidth to the public network, measured in Mbps (Mega bit per second). Value range:  [0, 100]. Default to 0 Mbps.
        :param pulumi.Input[str] io_optimized: It has been deprecated on instance resource. All the launched alicloud instances will be I/O optimized.
        :param pulumi.Input[bool] is_outdated: Whether to use outdated instance type. Default to false.
        :param pulumi.Input[str] key_name: The name of key pair that can login ECS instance successfully without password. If it is specified, the password would be invalid.
        :param pulumi.Input[str] kms_encrypted_password: An KMS encrypts password used to an instance. If the `password` is filled in, this field will be ignored. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[dict] kms_encryption_context: An KMS encryption context used to decrypt `kms_encrypted_password` before creating or updating an instance with `kms_encrypted_password`. See [Encryption Context](https://www.alibabacloud.com/help/doc-detail/42975.htm). It is valid when `kms_encrypted_password` is set. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[str] password: Password to an instance is a string of 8 to 30 characters. It must contain uppercase/lowercase letters and numerals, but cannot contain special symbols. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[float] period: The duration that you will buy the resource, in month. It is valid when `instance_charge_type` is `PrePaid`. Default to 1. Valid values:
               - [1-9, 12, 24, 36, 48, 60] when `period_unit` in "Month"
               - [1-3] when `period_unit` in "Week"
        :param pulumi.Input[str] period_unit: The duration unit that you will buy the resource. It is valid when `instance_charge_type` is 'PrePaid'. Valid value: ["Week", "Month"]. Default to "Month".
        :param pulumi.Input[str] private_ip: Instance private IP address can be specified when you creating new instance. It is valid when `vswitch_id` is specified. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[str] renewal_status: Whether to renew an ECS instance automatically or not. It is valid when `instance_charge_type` is `PrePaid`. Default to "Normal". Valid values:
               - `AutoRenewal`: Enable auto renewal.
               - `Normal`: Disable auto renewal.
               - `NotRenewal`: No renewal any longer. After you specify this value, Alibaba Cloud stop sending notification of instance expiry, and only gives a brief reminder on the third day before the instance expiry.
        :param pulumi.Input[str] resource_group_id: The Id of resource group which the instance belongs.
        :param pulumi.Input[str] role_name: Instance RAM role name. The name is provided and maintained by RAM. You can use `ram.Role` to create a new one.
        :param pulumi.Input[str] security_enhancement_strategy: The security enhancement strategy.
               - Active: Enable security enhancement strategy, it only works on system images.
               - Deactive: Disable security enhancement strategy, it works on all images.
        :param pulumi.Input[list] security_groups: A list of security group ids to associate with.
        :param pulumi.Input[float] spot_price_limit: The hourly price threshold of a instance, and it takes effect only when parameter 'spot_strategy' is 'SpotWithPriceLimit'. Three decimals is allowed at most.
        :param pulumi.Input[str] spot_strategy: The spot strategy of a Pay-As-You-Go instance, and it takes effect only when parameter `instance_charge_type` is 'PostPaid'. Value range:
               - NoSpot: A regular Pay-As-You-Go instance.
               - SpotWithPriceLimit: A price threshold for a spot instance
               - SpotAsPriceGo: A price that is based on the highest Pay-As-You-Go instance
        :param pulumi.Input[str] system_disk_category: Valid values are `ephemeral_ssd`, `cloud_efficiency`, `cloud_ssd`, `cloud_essd`, `cloud`. `cloud` only is used to some none I/O optimized instance. Default to `cloud_efficiency`.
        :param pulumi.Input[float] system_disk_size: Size of the system disk, measured in GiB. Value range: [20, 500]. The specified value must be equal to or greater than max{20, Imagesize}. Default value: max{40, ImageSize}. ECS instance's system disk can be reset when replacing system disk. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[dict] tags: A mapping of tags to assign to the resource.
               - Key: It can be up to 64 characters in length. It cannot begin with "aliyun", "acs:", "http://", or "https://". It cannot be a null string.
               - Value: It can be up to 128 characters in length. It cannot begin with "aliyun", "acs:", "http://", or "https://". It can be a null string.
        :param pulumi.Input[str] user_data: User-defined data to customize the startup behaviors of an ECS instance and to pass data into an ECS instance. From version 1.60.0, it can be update in-place. If updated, the instance will reboot to make the change take effect. Note: Not all of changes will take effect and it depends on [cloud-init module type](https://cloudinit.readthedocs.io/en/latest/topics/modules.html).
        :param pulumi.Input[dict] volume_tags: A mapping of tags to assign to the devices created by the instance at launch time.
               - Key: It can be up to 64 characters in length. It cannot begin with "aliyun", "acs:", "http://", or "https://". It cannot be a null string.
               - Value: It can be up to 128 characters in length. It cannot begin with "aliyun", "acs:", "http://", or "https://". It can be a null string.
        :param pulumi.Input[str] vswitch_id: The virtual switch ID to launch in VPC. This parameter must be set unless you can create classic network instances. When it is changed, the instance will reboot to make the change take effect.
        
        The **data_disks** object supports the following:
        
          * `category` (`pulumi.Input[str]`) - The category of the disk:
            - `cloud`: The general cloud disk.
            - `cloud_efficiency`: The efficiency cloud disk.
            - `cloud_ssd`: The SSD cloud disk.
            - `cloud_essd`: The ESSD cloud disk.
            - `ephemeral_ssd`: The local SSD disk.
            Default to `cloud_efficiency`.
            * `encrypted` -(Optional, Bool, ForceNew) Encrypted the data in this disk.
          * `delete_with_instance` (`pulumi.Input[bool]`) - Delete this data disk when the instance is destroyed. It only works on cloud, cloud_efficiency, cloud_essd, cloud_ssd disk. If the category of this data disk was ephemeral_ssd, please don't set this param.
          * `description` (`pulumi.Input[str]`) - The description of the data disk.
          * `encrypted` (`pulumi.Input[bool]`)
          * `name` (`pulumi.Input[str]`) - The name of the data disk.
          * `size` (`pulumi.Input[float]`) - The size of the data disk.
            - cloud：[5, 2000]
            - cloud_efficiency：[20, 32768]
            - cloud_ssd：[20, 32768]
            - cloud_essd：[20, 32768]
            - ephemeral_ssd: [5, 800]
          * `snapshot_id` (`pulumi.Input[str]`) - The snapshot ID used to initialize the data disk. If the size specified by snapshot is greater that the size of the disk, use the size specified by snapshot as the size of the data disk.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-alicloud/blob/master/website/docs/r/instance.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['allocate_public_ip'] = allocate_public_ip
            __props__['auto_release_time'] = auto_release_time
            __props__['auto_renew_period'] = auto_renew_period
            __props__['availability_zone'] = availability_zone
            __props__['credit_specification'] = credit_specification
            __props__['data_disks'] = data_disks
            __props__['deletion_protection'] = deletion_protection
            __props__['description'] = description
            __props__['dry_run'] = dry_run
            __props__['force_delete'] = force_delete
            __props__['host_name'] = host_name
            if image_id is None:
                raise TypeError("Missing required property 'image_id'")
            __props__['image_id'] = image_id
            __props__['include_data_disks'] = include_data_disks
            __props__['instance_charge_type'] = instance_charge_type
            __props__['instance_name'] = instance_name
            if instance_type is None:
                raise TypeError("Missing required property 'instance_type'")
            __props__['instance_type'] = instance_type
            __props__['internet_charge_type'] = internet_charge_type
            __props__['internet_max_bandwidth_in'] = internet_max_bandwidth_in
            __props__['internet_max_bandwidth_out'] = internet_max_bandwidth_out
            __props__['io_optimized'] = io_optimized
            __props__['is_outdated'] = is_outdated
            __props__['key_name'] = key_name
            __props__['kms_encrypted_password'] = kms_encrypted_password
            __props__['kms_encryption_context'] = kms_encryption_context
            __props__['password'] = password
            __props__['period'] = period
            __props__['period_unit'] = period_unit
            __props__['private_ip'] = private_ip
            __props__['renewal_status'] = renewal_status
            __props__['resource_group_id'] = resource_group_id
            __props__['role_name'] = role_name
            __props__['security_enhancement_strategy'] = security_enhancement_strategy
            if security_groups is None:
                raise TypeError("Missing required property 'security_groups'")
            __props__['security_groups'] = security_groups
            __props__['spot_price_limit'] = spot_price_limit
            __props__['spot_strategy'] = spot_strategy
            __props__['subnet_id'] = subnet_id
            __props__['system_disk_category'] = system_disk_category
            __props__['system_disk_size'] = system_disk_size
            __props__['tags'] = tags
            __props__['user_data'] = user_data
            __props__['volume_tags'] = volume_tags
            __props__['vswitch_id'] = vswitch_id
            __props__['public_ip'] = None
            __props__['status'] = None
        super(Instance, __self__).__init__(
            'alicloud:ecs/instance:Instance',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, allocate_public_ip=None, auto_release_time=None, auto_renew_period=None, availability_zone=None, credit_specification=None, data_disks=None, deletion_protection=None, description=None, dry_run=None, force_delete=None, host_name=None, image_id=None, include_data_disks=None, instance_charge_type=None, instance_name=None, instance_type=None, internet_charge_type=None, internet_max_bandwidth_in=None, internet_max_bandwidth_out=None, io_optimized=None, is_outdated=None, key_name=None, kms_encrypted_password=None, kms_encryption_context=None, password=None, period=None, period_unit=None, private_ip=None, public_ip=None, renewal_status=None, resource_group_id=None, role_name=None, security_enhancement_strategy=None, security_groups=None, spot_price_limit=None, spot_strategy=None, status=None, subnet_id=None, system_disk_category=None, system_disk_size=None, tags=None, user_data=None, volume_tags=None, vswitch_id=None):
        """
        Get an existing Instance resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] allocate_public_ip: It has been deprecated from version "1.7.0". Setting "internet_max_bandwidth_out" larger than 0 can allocate a public ip address for an instance.
        :param pulumi.Input[str] auto_release_time: The automatic release time of the `PostPaid` instance. 
               The time follows the ISO 8601 standard and is in UTC time. Format: yyyy-MM-ddTHH:mm:ssZ. It must be at least half an hour later than the current time and less than 3 years since the current time.
               Set it to null can cancel automatic release attribute and the ECS instance will not be released automatically.
        :param pulumi.Input[float] auto_renew_period: Auto renewal period of an instance, in the unit of month. It is valid when `instance_charge_type` is `PrePaid`. Default to 1. Valid value:
               - [1, 2, 3, 6, 12] when `period_unit` in "Month"
               - [1, 2, 3] when `period_unit` in "Week"
        :param pulumi.Input[str] availability_zone: The Zone to start the instance in. It is ignored and will be computed when set `vswitch_id`.
        :param pulumi.Input[str] credit_specification: Performance mode of the t5 burstable instance. Valid values: 'Standard', 'Unlimited'.
        :param pulumi.Input[list] data_disks: The list of data disks created with instance.
        :param pulumi.Input[bool] deletion_protection: Whether enable the deletion protection or not.
               - true: Enable deletion protection.
               - false: Disable deletion protection.
        :param pulumi.Input[str] description: The description of the data disk.
        :param pulumi.Input[bool] dry_run: Specifies whether to send a dry-run request. Default to false. 
               - true: Only a dry-run request is sent and no instance is created. The system checks whether the required parameters are set, and validates the request format, service permissions, and available ECS instances. If the validation fails, the corresponding error code is returned. If the validation succeeds, the `DryRunOperation` error code is returned.
               - false: A request is sent. If the validation succeeds, the instance is created.
        :param pulumi.Input[bool] force_delete: If it is true, the "PrePaid" instance will be change to "PostPaid" and then deleted forcibly.
               However, because of changing instance charge type has CPU core count quota limitation, so strongly recommand that "Don't modify instance charge type frequentlly in one month".
        :param pulumi.Input[str] host_name: Host name of the ECS, which is a string of at least two characters. “hostname” cannot start or end with “.” or “-“. In addition, two or more consecutive “.” or “-“ symbols are not allowed. On Windows, the host name can contain a maximum of 15 characters, which can be a combination of uppercase/lowercase letters, numerals, and “-“. The host name cannot contain dots (“.”) or contain only numeric characters. When it is changed, the instance will reboot to make the change take effect.
               On other OSs such as Linux, the host name can contain a maximum of 30 characters, which can be segments separated by dots (“.”), where each segment can contain uppercase/lowercase letters, numerals, or “_“. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[str] image_id: The Image to use for the instance. ECS instance's image can be replaced via changing 'image_id'. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[bool] include_data_disks: Whether to change instance disks charge type when changing instance charge type.
        :param pulumi.Input[str] instance_charge_type: Valid values are `PrePaid`, `PostPaid`, The default is `PostPaid`.
        :param pulumi.Input[str] instance_type: The type of instance to start. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[str] internet_charge_type: Internet charge type of the instance, Valid values are `PayByBandwidth`, `PayByTraffic`. Default is `PayByTraffic`. At present, 'PrePaid' instance cannot change the value to "PayByBandwidth" from "PayByTraffic".
        :param pulumi.Input[float] internet_max_bandwidth_in: Maximum incoming bandwidth from the public network, measured in Mbps (Mega bit per second). Value range: [1, 200]. If this value is not specified, then automatically sets it to 200 Mbps.
        :param pulumi.Input[float] internet_max_bandwidth_out: Maximum outgoing bandwidth to the public network, measured in Mbps (Mega bit per second). Value range:  [0, 100]. Default to 0 Mbps.
        :param pulumi.Input[str] io_optimized: It has been deprecated on instance resource. All the launched alicloud instances will be I/O optimized.
        :param pulumi.Input[bool] is_outdated: Whether to use outdated instance type. Default to false.
        :param pulumi.Input[str] key_name: The name of key pair that can login ECS instance successfully without password. If it is specified, the password would be invalid.
        :param pulumi.Input[str] kms_encrypted_password: An KMS encrypts password used to an instance. If the `password` is filled in, this field will be ignored. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[dict] kms_encryption_context: An KMS encryption context used to decrypt `kms_encrypted_password` before creating or updating an instance with `kms_encrypted_password`. See [Encryption Context](https://www.alibabacloud.com/help/doc-detail/42975.htm). It is valid when `kms_encrypted_password` is set. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[str] password: Password to an instance is a string of 8 to 30 characters. It must contain uppercase/lowercase letters and numerals, but cannot contain special symbols. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[float] period: The duration that you will buy the resource, in month. It is valid when `instance_charge_type` is `PrePaid`. Default to 1. Valid values:
               - [1-9, 12, 24, 36, 48, 60] when `period_unit` in "Month"
               - [1-3] when `period_unit` in "Week"
        :param pulumi.Input[str] period_unit: The duration unit that you will buy the resource. It is valid when `instance_charge_type` is 'PrePaid'. Valid value: ["Week", "Month"]. Default to "Month".
        :param pulumi.Input[str] private_ip: Instance private IP address can be specified when you creating new instance. It is valid when `vswitch_id` is specified. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[str] public_ip: The instance public ip.
        :param pulumi.Input[str] renewal_status: Whether to renew an ECS instance automatically or not. It is valid when `instance_charge_type` is `PrePaid`. Default to "Normal". Valid values:
               - `AutoRenewal`: Enable auto renewal.
               - `Normal`: Disable auto renewal.
               - `NotRenewal`: No renewal any longer. After you specify this value, Alibaba Cloud stop sending notification of instance expiry, and only gives a brief reminder on the third day before the instance expiry.
        :param pulumi.Input[str] resource_group_id: The Id of resource group which the instance belongs.
        :param pulumi.Input[str] role_name: Instance RAM role name. The name is provided and maintained by RAM. You can use `ram.Role` to create a new one.
        :param pulumi.Input[str] security_enhancement_strategy: The security enhancement strategy.
               - Active: Enable security enhancement strategy, it only works on system images.
               - Deactive: Disable security enhancement strategy, it works on all images.
        :param pulumi.Input[list] security_groups: A list of security group ids to associate with.
        :param pulumi.Input[float] spot_price_limit: The hourly price threshold of a instance, and it takes effect only when parameter 'spot_strategy' is 'SpotWithPriceLimit'. Three decimals is allowed at most.
        :param pulumi.Input[str] spot_strategy: The spot strategy of a Pay-As-You-Go instance, and it takes effect only when parameter `instance_charge_type` is 'PostPaid'. Value range:
               - NoSpot: A regular Pay-As-You-Go instance.
               - SpotWithPriceLimit: A price threshold for a spot instance
               - SpotAsPriceGo: A price that is based on the highest Pay-As-You-Go instance
        :param pulumi.Input[str] status: The instance status.
        :param pulumi.Input[str] system_disk_category: Valid values are `ephemeral_ssd`, `cloud_efficiency`, `cloud_ssd`, `cloud_essd`, `cloud`. `cloud` only is used to some none I/O optimized instance. Default to `cloud_efficiency`.
        :param pulumi.Input[float] system_disk_size: Size of the system disk, measured in GiB. Value range: [20, 500]. The specified value must be equal to or greater than max{20, Imagesize}. Default value: max{40, ImageSize}. ECS instance's system disk can be reset when replacing system disk. When it is changed, the instance will reboot to make the change take effect.
        :param pulumi.Input[dict] tags: A mapping of tags to assign to the resource.
               - Key: It can be up to 64 characters in length. It cannot begin with "aliyun", "acs:", "http://", or "https://". It cannot be a null string.
               - Value: It can be up to 128 characters in length. It cannot begin with "aliyun", "acs:", "http://", or "https://". It can be a null string.
        :param pulumi.Input[str] user_data: User-defined data to customize the startup behaviors of an ECS instance and to pass data into an ECS instance. From version 1.60.0, it can be update in-place. If updated, the instance will reboot to make the change take effect. Note: Not all of changes will take effect and it depends on [cloud-init module type](https://cloudinit.readthedocs.io/en/latest/topics/modules.html).
        :param pulumi.Input[dict] volume_tags: A mapping of tags to assign to the devices created by the instance at launch time.
               - Key: It can be up to 64 characters in length. It cannot begin with "aliyun", "acs:", "http://", or "https://". It cannot be a null string.
               - Value: It can be up to 128 characters in length. It cannot begin with "aliyun", "acs:", "http://", or "https://". It can be a null string.
        :param pulumi.Input[str] vswitch_id: The virtual switch ID to launch in VPC. This parameter must be set unless you can create classic network instances. When it is changed, the instance will reboot to make the change take effect.
        
        The **data_disks** object supports the following:
        
          * `category` (`pulumi.Input[str]`) - The category of the disk:
            - `cloud`: The general cloud disk.
            - `cloud_efficiency`: The efficiency cloud disk.
            - `cloud_ssd`: The SSD cloud disk.
            - `cloud_essd`: The ESSD cloud disk.
            - `ephemeral_ssd`: The local SSD disk.
            Default to `cloud_efficiency`.
            * `encrypted` -(Optional, Bool, ForceNew) Encrypted the data in this disk.
          * `delete_with_instance` (`pulumi.Input[bool]`) - Delete this data disk when the instance is destroyed. It only works on cloud, cloud_efficiency, cloud_essd, cloud_ssd disk. If the category of this data disk was ephemeral_ssd, please don't set this param.
          * `description` (`pulumi.Input[str]`) - The description of the data disk.
          * `encrypted` (`pulumi.Input[bool]`)
          * `name` (`pulumi.Input[str]`) - The name of the data disk.
          * `size` (`pulumi.Input[float]`) - The size of the data disk.
            - cloud：[5, 2000]
            - cloud_efficiency：[20, 32768]
            - cloud_ssd：[20, 32768]
            - cloud_essd：[20, 32768]
            - ephemeral_ssd: [5, 800]
          * `snapshot_id` (`pulumi.Input[str]`) - The snapshot ID used to initialize the data disk. If the size specified by snapshot is greater that the size of the disk, use the size specified by snapshot as the size of the data disk.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-alicloud/blob/master/website/docs/r/instance.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["allocate_public_ip"] = allocate_public_ip
        __props__["auto_release_time"] = auto_release_time
        __props__["auto_renew_period"] = auto_renew_period
        __props__["availability_zone"] = availability_zone
        __props__["credit_specification"] = credit_specification
        __props__["data_disks"] = data_disks
        __props__["deletion_protection"] = deletion_protection
        __props__["description"] = description
        __props__["dry_run"] = dry_run
        __props__["force_delete"] = force_delete
        __props__["host_name"] = host_name
        __props__["image_id"] = image_id
        __props__["include_data_disks"] = include_data_disks
        __props__["instance_charge_type"] = instance_charge_type
        __props__["instance_name"] = instance_name
        __props__["instance_type"] = instance_type
        __props__["internet_charge_type"] = internet_charge_type
        __props__["internet_max_bandwidth_in"] = internet_max_bandwidth_in
        __props__["internet_max_bandwidth_out"] = internet_max_bandwidth_out
        __props__["io_optimized"] = io_optimized
        __props__["is_outdated"] = is_outdated
        __props__["key_name"] = key_name
        __props__["kms_encrypted_password"] = kms_encrypted_password
        __props__["kms_encryption_context"] = kms_encryption_context
        __props__["password"] = password
        __props__["period"] = period
        __props__["period_unit"] = period_unit
        __props__["private_ip"] = private_ip
        __props__["public_ip"] = public_ip
        __props__["renewal_status"] = renewal_status
        __props__["resource_group_id"] = resource_group_id
        __props__["role_name"] = role_name
        __props__["security_enhancement_strategy"] = security_enhancement_strategy
        __props__["security_groups"] = security_groups
        __props__["spot_price_limit"] = spot_price_limit
        __props__["spot_strategy"] = spot_strategy
        __props__["status"] = status
        __props__["subnet_id"] = subnet_id
        __props__["system_disk_category"] = system_disk_category
        __props__["system_disk_size"] = system_disk_size
        __props__["tags"] = tags
        __props__["user_data"] = user_data
        __props__["volume_tags"] = volume_tags
        __props__["vswitch_id"] = vswitch_id
        return Instance(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

__config__ = pulumi.Config('alicloud')

access_key = __config__.get('accessKey') or utilities.get_env('ALICLOUD_ACCESS_KEY')
"""
The access key for API operations. You can retrieve this from the 'Security Management' section of the Alibaba Cloud
console.
"""

account_id = __config__.get('accountId') or utilities.get_env('ALICLOUD_ACCOUNT_ID')
"""
The account ID for some service API operations. You can retrieve this from the 'Security Settings' section of the
Alibaba Cloud console.
"""

assume_role = __config__.get('assumeRole')

configuration_source = __config__.get('configurationSource')
"""
Use this to mark a terraform configuration file source.
"""

ecs_role_name = __config__.get('ecsRoleName') or utilities.get_env('ALICLOUD_ECS_ROLE_NAME')
"""
The RAM Role Name attached on a ECS instance for API operations. You can retrieve this from the 'Access Control' section
of the Alibaba Cloud console.
"""

endpoints = __config__.get('endpoints')

fc = __config__.get('fc')

log_endpoint = __config__.get('logEndpoint')

mns_endpoint = __config__.get('mnsEndpoint')

ots_instance_name = __config__.get('otsInstanceName')

profile = __config__.get('profile') or utilities.get_env('ALICLOUD_PROFILE')
"""
The profile for API operations. If not set, the default profile created with `aliyun configure` will be used.
"""

region = __config__.get('region') or utilities.get_env('ALICLOUD_REGION')
"""
The region where Alibaba Cloud operations will take place. Examples are cn-beijing, cn-hangzhou, eu-central-1, etc.
"""

secret_key = __config__.get('secretKey') or utilities.get_env('ALICLOUD_SECRET_KEY')
"""
The secret key for API operations. You can retrieve this from the 'Security Management' section of the Alibaba Cloud
console.
"""

security_token = __config__.get('securityToken') or utilities.get_env('ALICLOUD_SECURITY_TOKEN')
"""
security token. A security token is only required if you are using Security Token Service.
"""

shared_credentials_file = __config__.get('sharedCredentialsFile') or utilities.get_env('ALICLOUD_SHARED_CREDENTIALS_FILE')
"""
The path to the shared credentials file. If not set this defaults to ~/.aliyun/config.json
"""

skip_region_validation = __config__.get('skipRegionValidation')
"""
Skip static validation of region ID. Used by users of alternative AlibabaCloud-like APIs or users w/ access to regions
that are not public (yet).
"""




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class GetDomainExtensionsResult:
    """
    A collection of values returned by getDomainExtensions.
    """
    def __init__(__self__, extensions=None, frontend_port=None, ids=None, load_balancer_id=None, output_file=None, id=None):
        if extensions and not isinstance(extensions, list):
            raise TypeError("Expected argument 'extensions' to be a list")
        __self__.extensions = extensions
        """
        A list of SLB domain extension. Each element contains the following attributes:
        """
        if frontend_port and not isinstance(frontend_port, float):
            raise TypeError("Expected argument 'frontend_port' to be a float")
        __self__.frontend_port = frontend_port
        if ids and not isinstance(ids, list):
            raise TypeError("Expected argument 'ids' to be a list")
        __self__.ids = ids
        if load_balancer_id and not isinstance(load_balancer_id, str):
            raise TypeError("Expected argument 'load_balancer_id' to be a str")
        __self__.load_balancer_id = load_balancer_id
        if output_file and not isinstance(output_file, str):
            raise TypeError("Expected argument 'output_file' to be a str")
        __self__.output_file = output_file
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        __self__.id = id
        """
        id is the provider-assigned unique ID for this managed resource.
        """
class AwaitableGetDomainExtensionsResult(GetDomainExtensionsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetDomainExtensionsResult(
            extensions=self.extensions,
            frontend_port=self.frontend_port,
            ids=self.ids,
            load_balancer_id=self.load_balancer_id,
            output_file=self.output_file,
            id=self.id)

def get_domain_extensions(frontend_port=None,ids=None,load_balancer_id=None,output_file=None,opts=None):
    """
    This data source provides the domain extensions associated with a server load balancer listener.
    
    > **NOTE:** Available in 1.60.0+
    
    :param float frontend_port: The frontend port used by the HTTPS listener of the SLB instance. Valid values: 1–65535.
    :param list ids: IDs of the SLB domain extensions.
    :param str load_balancer_id: The ID of the SLB instance.

    > This content is derived from https://github.com/terraform-providers/terraform-provider-alicloud/blob/master/website/docs/d/slb_domain_extensions.html.markdown.
    """
    __args__ = dict()

    __args__['frontendPort'] = frontend_port
    __args__['ids'] = ids
    __args__['loadBalancerId'] = load_balancer_id
    __args__['outputFile'] = output_file
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('alicloud:slb/getDomainExtensions:getDomainExtensions', __args__, opts=opts).value

    return AwaitableGetDomainExtensionsResult(
        extensions=__ret__.get('extensions'),
        frontend_port=__ret__.get('frontendPort'),
        ids=__ret__.get('ids'),
        load_balancer_id=__ret__.get('loadBalancerId'),
        output_file=__ret__.get('outputFile'),
        id=__ret__.get('id'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class BandwidthPackage(pulumi.CustomResource):
    bandwidth: pulumi.Output[float]
    """
    The bandwidth in Mbps of the bandwidth package. Cannot be less than 2Mbps.
    """
    charge_type: pulumi.Output[str]
    """
    The billing method. Valid value: PostPaid | PrePaid. Default to PostPaid. If set to PrePaid, the bandwidth package can't be deleted before expired time.
    """
    description: pulumi.Output[str]
    """
    The description of the bandwidth package. Default to null.
    """
    expired_time: pulumi.Output[str]
    """
    The time of the bandwidth package to expire.
    """
    geographic_region_ids: pulumi.Output[list]
    """
    List of the two areas to connect. Valid value: China | North-America | Asia-Pacific | Europe | Middle-East.
    """
    name: pulumi.Output[str]
    """
    The name of the bandwidth package. Defaults to null.
    """
    period: pulumi.Output[float]
    """
    The purchase period in month. Valid value: 1, 2, 3, 6, 12. Default to 1.
    """
    status: pulumi.Output[str]
    """
    The status of the bandwidth, including "InUse" and "Idle".
    """
    def __init__(__self__, resource_name, opts=None, bandwidth=None, charge_type=None, description=None, geographic_region_ids=None, name=None, period=None, __props__=None, __name__=None, __opts__=None):
        """
        Provides a CEN bandwidth package resource. The CEN bandwidth package is an abstracted object that includes an interconnection bandwidth and interconnection areas. To buy a bandwidth package, you must specify the areas to connect. An area consists of one or more Alibaba Cloud regions. The areas in CEN include Mainland China, Asia Pacific, North America, and Europe.
        
        For information about CEN and how to use it, see [Manage bandwidth packages](https://www.alibabacloud.com/help/doc-detail/65982.htm).
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[float] bandwidth: The bandwidth in Mbps of the bandwidth package. Cannot be less than 2Mbps.
        :param pulumi.Input[str] charge_type: The billing method. Valid value: PostPaid | PrePaid. Default to PostPaid. If set to PrePaid, the bandwidth package can't be deleted before expired time.
        :param pulumi.Input[str] description: The description of the bandwidth package. Default to null.
        :param pulumi.Input[list] geographic_region_ids: List of the two areas to connect. Valid value: China | North-America | Asia-Pacific | Europe | Middle-East.
        :param pulumi.Input[str] name: The name of the bandwidth package. Defaults to null.
        :param pulumi.Input[float] period: The purchase period in month. Valid value: 1, 2, 3, 6, 12. Default to 1.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-alicloud/blob/master/website/docs/r/cen_bandwidth_package.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if bandwidth is None:
                raise TypeError("Missing required property 'bandwidth'")
            __props__['bandwidth'] = bandwidth
            __props__['charge_type'] = charge_type
            __props__['description'] = description
            if geographic_region_ids is None:
                raise TypeError("Missing required property 'geographic_region_ids'")
            __props__['geographic_region_ids'] = geographic_region_ids
            __props__['name'] = name
            __props__['period'] = period
            __props__['expired_time'] = None
            __props__['status'] = None
        super(BandwidthPackage, __self__).__init__(
            'alicloud:cen/bandwidthPackage:BandwidthPackage',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, bandwidth=None, charge_type=None, description=None, expired_time=None, geographic_region_ids=None, name=None, period=None, status=None):
        """
        Get an existing BandwidthPackage resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[float] bandwidth: The bandwidth in Mbps of the bandwidth package. Cannot be less than 2Mbps.
        :param pulumi.Input[str] charge_type: The billing method. Valid value: PostPaid | PrePaid. Default to PostPaid. If set to PrePaid, the bandwidth package can't be deleted before expired time.
        :param pulumi.Input[str] description: The description of the bandwidth package. Default to null.
        :param pulumi.Input[str] expired_time: The time of the bandwidth package to expire.
        :param pulumi.Input[list] geographic_region_ids: List of the two areas to connect. Valid value: China | North-America | Asia-Pacific | Europe | Middle-East.
        :param pulumi.Input[str] name: The name of the bandwidth package. Defaults to null.
        :param pulumi.Input[float] period: The purchase period in month. Valid value: 1, 2, 3, 6, 12. Default to 1.
        :param pulumi.Input[str] status: The status of the bandwidth, including "InUse" and "Idle".

        > This content is derived from https://github.com/terraform-providers/terraform-provider-alicloud/blob/master/website/docs/r/cen_bandwidth_package.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["bandwidth"] = bandwidth
        __props__["charge_type"] = charge_type
        __props__["description"] = description
        __props__["expired_time"] = expired_time
        __props__["geographic_region_ids"] = geographic_region_ids
        __props__["name"] = name
        __props__["period"] = period
        __props__["status"] = status
        return BandwidthPackage(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class DdosBgpInstance(pulumi.CustomResource):
    bandwidth: pulumi.Output[float]
    """
    Elastic defend bandwidth of the instance. This value must be larger than the base defend bandwidth. Valid values: 51,91,101,201,301. The unit is Gbps.
    """
    base_bandwidth: pulumi.Output[float]
    """
    Base defend bandwidth of the instance. Valid values: 20. The unit is Gbps. Default to `20`.
    """
    ip_count: pulumi.Output[float]
    """
    IP count of the instance. Valid values: 100.
    """
    ip_type: pulumi.Output[str]
    """
    IP version of the instance. Valid values: IPv4,IPv6.
    """
    name: pulumi.Output[str]
    """
    Name of the instance. This name can have a string of 1 to 63 characters.
    """
    period: pulumi.Output[float]
    """
    The duration that you will buy Ddosbgp instance (in month). Valid values: [1~9], 12, 24, 36. Default to 12. At present, the provider does not support modify "period".
    """
    type: pulumi.Output[str]
    """
    Type of the instance. Valid values: Enterprise,Professional. Default to `Enterprise`  
    """
    def __init__(__self__, resource_name, opts=None, bandwidth=None, base_bandwidth=None, ip_count=None, ip_type=None, name=None, period=None, type=None, __props__=None, __name__=None, __opts__=None):
        """
        Anti-DDoS Advanced instance resource. "Ddosbgp" is the short term of this product.
        
        > **NOTE:** The endpoint of bssopenapi used only support "business.aliyuncs.com" at present.
        
        > **NOTE:** Available in 1.57.0+ .
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[float] bandwidth: Elastic defend bandwidth of the instance. This value must be larger than the base defend bandwidth. Valid values: 51,91,101,201,301. The unit is Gbps.
        :param pulumi.Input[float] base_bandwidth: Base defend bandwidth of the instance. Valid values: 20. The unit is Gbps. Default to `20`.
        :param pulumi.Input[float] ip_count: IP count of the instance. Valid values: 100.
        :param pulumi.Input[str] ip_type: IP version of the instance. Valid values: IPv4,IPv6.
        :param pulumi.Input[str] name: Name of the instance. This name can have a string of 1 to 63 characters.
        :param pulumi.Input[float] period: The duration that you will buy Ddosbgp instance (in month). Valid values: [1~9], 12, 24, 36. Default to 12. At present, the provider does not support modify "period".
        :param pulumi.Input[str] type: Type of the instance. Valid values: Enterprise,Professional. Default to `Enterprise`  

        > This content is derived from https://github.com/terraform-providers/terraform-provider-alicloud/blob/master/website/docs/r/ddosbgp_instance.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if bandwidth is None:
                raise TypeError("Missing required property 'bandwidth'")
            __props__['bandwidth'] = bandwidth
            __props__['base_bandwidth'] = base_bandwidth
            if ip_count is None:
                raise TypeError("Missing required property 'ip_count'")
            __props__['ip_count'] = ip_count
            if ip_type is None:
                raise TypeError("Missing required property 'ip_type'")
            __props__['ip_type'] = ip_type
            __props__['name'] = name
            __props__['period'] = period
            __props__['type'] = type
        super(DdosBgpInstance, __self__).__init__(
            'alicloud:dns/ddosBgpInstance:DdosBgpInstance',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, bandwidth=None, base_bandwidth=None, ip_count=None, ip_type=None, name=None, period=None, type=None):
        """
        Get an existing DdosBgpInstance resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[float] bandwidth: Elastic defend bandwidth of the instance. This value must be larger than the base defend bandwidth. Valid values: 51,91,101,201,301. The unit is Gbps.
        :param pulumi.Input[float] base_bandwidth: Base defend bandwidth of the instance. Valid values: 20. The unit is Gbps. Default to `20`.
        :param pulumi.Input[float] ip_count: IP count of the instance. Valid values: 100.
        :param pulumi.Input[str] ip_type: IP version of the instance. Valid values: IPv4,IPv6.
        :param pulumi.Input[str] name: Name of the instance. This name can have a string of 1 to 63 characters.
        :param pulumi.Input[float] period: The duration that you will buy Ddosbgp instance (in month). Valid values: [1~9], 12, 24, 36. Default to 12. At present, the provider does not support modify "period".
        :param pulumi.Input[str] type: Type of the instance. Valid values: Enterprise,Professional. Default to `Enterprise`  

        > This content is derived from https://github.com/terraform-providers/terraform-provider-alicloud/blob/master/website/docs/r/ddosbgp_instance.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["bandwidth"] = bandwidth
        __props__["base_bandwidth"] = base_bandwidth
        __props__["ip_count"] = ip_count
        __props__["ip_type"] = ip_type
        __props__["name"] = name
        __props__["period"] = period
        __props__["type"] = type
        return DdosBgpInstance(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Database(pulumi.CustomResource):
    character_set: pulumi.Output[str]
    """
    Character set. The value range is limited to the following:
    - MySQL: [ utf8, gbk, latin1, utf8mb4 ] \(`utf8mb4` only supports versions 5.5 and 5.6\).
    - SQLServer: [ Chinese_PRC_CI_AS, Chinese_PRC_CS_AS, SQL_Latin1_General_CP1_CI_AS, SQL_Latin1_General_CP1_CS_AS, Chinese_PRC_BIN ]
    - PostgreSQL: [ KOI8U、UTF8、WIN866、WIN874、WIN1250、WIN1251、WIN1252、WIN1253、WIN1254、WIN1255、WIN1256、WIN1257、WIN1258、EUC_CN、EUC_KR、EUC_TW、EUC_JP、EUC_JIS_2004、KOI8R、MULE_INTERNAL、LATIN1、LATIN2、LATIN3、LATIN4、LATIN5、LATIN6、LATIN7、LATIN8、LATIN9、LATIN10、ISO_8859_5、ISO_8859_6、ISO_8859_7、ISO_8859_8、SQL_ASCII ]
    """
    description: pulumi.Output[str]
    """
    Database description. It cannot begin with https://. It must start with a Chinese character or English letter. It can include Chinese and English characters, underlines (_), hyphens (-), and numbers. The length may be 2-256 characters.
    """
    instance_id: pulumi.Output[str]
    """
    The Id of instance that can run database.
    """
    name: pulumi.Output[str]
    """
    Name of the database requiring a uniqueness check. It may consist of lower case letters, numbers, and underlines, and must start with a letter
    and have no more than 64 characters.
    """
    def __init__(__self__, resource_name, opts=None, character_set=None, description=None, instance_id=None, name=None, __props__=None, __name__=None, __opts__=None):
        """
        Provides an RDS database resource. A DB database deployed in a DB instance. A DB instance can own multiple databases.
        
        > **NOTE:** This resource does not support creating 'PPAS' database. You have to login RDS instance to create manually.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] character_set: Character set. The value range is limited to the following:
               - MySQL: [ utf8, gbk, latin1, utf8mb4 ] \(`utf8mb4` only supports versions 5.5 and 5.6\).
               - SQLServer: [ Chinese_PRC_CI_AS, Chinese_PRC_CS_AS, SQL_Latin1_General_CP1_CI_AS, SQL_Latin1_General_CP1_CS_AS, Chinese_PRC_BIN ]
               - PostgreSQL: [ KOI8U、UTF8、WIN866、WIN874、WIN1250、WIN1251、WIN1252、WIN1253、WIN1254、WIN1255、WIN1256、WIN1257、WIN1258、EUC_CN、EUC_KR、EUC_TW、EUC_JP、EUC_JIS_2004、KOI8R、MULE_INTERNAL、LATIN1、LATIN2、LATIN3、LATIN4、LATIN5、LATIN6、LATIN7、LATIN8、LATIN9、LATIN10、ISO_8859_5、ISO_8859_6、ISO_8859_7、ISO_8859_8、SQL_ASCII ]
        :param pulumi.Input[str] description: Database description. It cannot begin with https://. It must start with a Chinese character or English letter. It can include Chinese and English characters, underlines (_), hyphens (-), and numbers. The length may be 2-256 characters.
        :param pulumi.Input[str] instance_id: The Id of instance that can run database.
        :param pulumi.Input[str] name: Name of the database requiring a uniqueness check. It may consist of lower case letters, numbers, and underlines, and must start with a letter
               and have no more than 64 characters.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-alicloud/blob/master/website/docs/r/db_database.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['character_set'] = character_set
            __props__['description'] = description
            if instance_id is None:
                raise TypeError("Missing required property 'instance_id'")
            __props__['instance_id'] = instance_id
            __props__['name'] = name
        super(Database, __self__).__init__(
            'alicloud:rds/database:Database',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, character_set=None, description=None, instance_id=None, name=None):
        """
        Get an existing Database resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] character_set: Character set. The value range is limited to the following:
               - MySQL: [ utf8, gbk, latin1, utf8mb4 ] \(`utf8mb4` only supports versions 5.5 and 5.6\).
               - SQLServer: [ Chinese_PRC_CI_AS, Chinese_PRC_CS_AS, SQL_Latin1_General_CP1_CI_AS, SQL_Latin1_General_CP1_CS_AS, Chinese_PRC_BIN ]
               - PostgreSQL: [ KOI8U、UTF8、WIN866、WIN874、WIN1250、WIN1251、WIN1252、WIN1253、WIN1254、WIN1255、WIN1256、WIN1257、WIN1258、EUC_CN、EUC_KR、EUC_TW、EUC_JP、EUC_JIS_2004、KOI8R、MULE_INTERNAL、LATIN1、LATIN2、LATIN3、LATIN4、LATIN5、LATIN6、LATIN7、LATIN8、LATIN9、LATIN10、ISO_8859_5、ISO_8859_6、ISO_8859_7、ISO_8859_8、SQL_ASCII ]
        :param pulumi.Input[str] description: Database description. It cannot begin with https://. It must start with a Chinese character or English letter. It can include Chinese and English characters, underlines (_), hyphens (-), and numbers. The length may be 2-256 characters.
        :param pulumi.Input[str] instance_id: The Id of instance that can run database.
        :param pulumi.Input[str] name: Name of the database requiring a uniqueness check. It may consist of lower case letters, numbers, and underlines, and must start with a letter
               and have no more than 64 characters.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-alicloud/blob/master/website/docs/r/db_database.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["character_set"] = character_set
        __props__["description"] = description
        __props__["instance_id"] = instance_id
        __props__["name"] = name
        return Database(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class GetListenersResult:
    """
    A collection of values returned by getListeners.
    """
    def __init__(__self__, description_regex=None, frontend_port=None, load_balancer_id=None, output_file=None, protocol=None, slb_listeners=None, id=None):
        if description_regex and not isinstance(description_regex, str):
            raise TypeError("Expected argument 'description_regex' to be a str")
        __self__.description_regex = description_regex
        if frontend_port and not isinstance(frontend_port, float):
            raise TypeError("Expected argument 'frontend_port' to be a float")
        __self__.frontend_port = frontend_port
        """
        Frontend port used to receive incoming traffic and distribute it to the backend servers.
        """
        if load_balancer_id and not isinstance(load_balancer_id, str):
            raise TypeError("Expected argument 'load_balancer_id' to be a str")
        __self__.load_balancer_id = load_balancer_id
        if output_file and not isinstance(output_file, str):
            raise TypeError("Expected argument 'output_file' to be a str")
        __self__.output_file = output_file
        if protocol and not isinstance(protocol, str):
            raise TypeError("Expected argument 'protocol' to be a str")
        __self__.protocol = protocol
        """
        Listener protocol. Possible values: `http`, `https`, `tcp` and `udp`.
        """
        if slb_listeners and not isinstance(slb_listeners, list):
            raise TypeError("Expected argument 'slb_listeners' to be a list")
        __self__.slb_listeners = slb_listeners
        """
        A list of SLB listeners. Each element contains the following attributes:
        """
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        __self__.id = id
        """
        id is the provider-assigned unique ID for this managed resource.
        """
class AwaitableGetListenersResult(GetListenersResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetListenersResult(
            description_regex=self.description_regex,
            frontend_port=self.frontend_port,
            load_balancer_id=self.load_balancer_id,
            output_file=self.output_file,
            protocol=self.protocol,
            slb_listeners=self.slb_listeners,
            id=self.id)

def get_listeners(description_regex=None,frontend_port=None,load_balancer_id=None,output_file=None,protocol=None,opts=None):
    """
    This data source provides the listeners related to a server load balancer of the current Alibaba Cloud user.
    
    :param str description_regex: A regex string to filter results by SLB listener description.
    :param float frontend_port: Filter listeners by the specified frontend port.
    :param str load_balancer_id: ID of the SLB with listeners.
    :param str protocol: Filter listeners by the specified protocol. Valid values: `http`, `https`, `tcp` and `udp`.

    > This content is derived from https://github.com/terraform-providers/terraform-provider-alicloud/blob/master/website/docs/d/slb_listeners.html.markdown.
    """
    __args__ = dict()

    __args__['descriptionRegex'] = description_regex
    __args__['frontendPort'] = frontend_port
    __args__['loadBalancerId'] = load_balancer_id
    __args__['outputFile'] = output_file
    __args__['protocol'] = protocol
    if opts is None:
        opts = pulumi.InvokeOptions()
    if opts.version is None:
        opts.version = utilities.get_version()
    __ret__ = pulumi.runtime.invoke('alicloud:slb/getListeners:getListeners', __args__, opts=opts).value

    return AwaitableGetListenersResult(
        description_regex=__ret__.get('descriptionRegex'),
        frontend_port=__ret__.get('frontendPort'),
        load_balancer_id=__ret__.get('loadBalancerId'),
        output_file=__ret__.get('outputFile'),
        protocol=__ret__.get('protocol'),
        slb_listeners=__ret__.get('slbListeners'),
        id=__ret__.get('id'))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class ImageExport(pulumi.CustomResource):
    image_id: pulumi.Output[str]
    """
    The source image ID.
    """
    oss_bucket: pulumi.Output[str]
    """
    Save the exported OSS bucket.
    """
    oss_prefix: pulumi.Output[str]
    """
    The prefix of your OSS Object. It can be composed of numbers or letters, and the character length is 1 ~ 30.
    """
    def __init__(__self__, resource_name, opts=None, image_id=None, oss_bucket=None, oss_prefix=None, __props__=None, __name__=None, __opts__=None):
        """
        Export a custom image to the OSS bucket in the same region as the custom image.
        
        > **NOTE:** If you create an ECS instance using a mirror image and create a system disk snapshot again, exporting a custom image created from the system disk snapshot is not supported.
        
        > **NOTE:** Support for exporting custom images that include data disk snapshot information in the image. The number of data disks cannot exceed 4 and the maximum capacity of a single data disk cannot exceed 500 GiB.
        
        > **NOTE:** Before exporting the image, you must authorize the cloud server ECS official service account to write OSS permissions through RAM.
        
        > **NOTE:** Available in 1.68.0+.
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] image_id: The source image ID.
        :param pulumi.Input[str] oss_bucket: Save the exported OSS bucket.
        :param pulumi.Input[str] oss_prefix: The prefix of your OSS Object. It can be composed of numbers or letters, and the character length is 1 ~ 30.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-alicloud/blob/master/website/docs/r/image_export.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if image_id is None:
                raise TypeError("Missing required property 'image_id'")
            __props__['image_id'] = image_id
            if oss_bucket is None:
                raise TypeError("Missing required property 'oss_bucket'")
            __props__['oss_bucket'] = oss_bucket
            __props__['oss_prefix'] = oss_prefix
        super(ImageExport, __self__).__init__(
            'alicloud:ecs/imageExport:ImageExport',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, image_id=None, oss_bucket=None, oss_prefix=None):
        """
        Get an existing ImageExport resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] image_id: The source image ID.
        :param pulumi.Input[str] oss_bucket: Save the exported OSS bucket.
        :param pulumi.Input[str] oss_prefix: The prefix of your OSS Object. It can be composed of numbers or letters, and the character length is 1 ~ 30.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-alicloud/blob/master/website/docs/r/image_export.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["image_id"] = image_id
        __props__["oss_bucket"] = oss_bucket
        __props__["oss_prefix"] = oss_prefix
        return ImageExport(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

# Export this package's modules as members:
from .ddos_bgp_instance import *
from .ddos_coo_instance import *
from .domain import *
from .group import *
from .record import *
from .get_domain_groups import *
from .get_domain_records import *
from .get_domains import *
from .get_groups import *
from .get_records import *
from .get_resolution_lines import *



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Image(pulumi.CustomResource):
    architecture: pulumi.Output[str]
    """
    Specifies the architecture of the system disk after you specify a data disk snapshot as the data source of the system disk for creating an image. Valid values: `i386` , Default is `x86_64`.
    """
    description: pulumi.Output[str]
    """
    The description of the image. It must be 2 to 256 characters in length and must not start with http:// or https://. Default value: null.
    """
    disk_device_mappings: pulumi.Output[list]
    """
    Description of the system with disks and snapshots under the image.
    
      * `device` (`str`) - Specifies the name of a disk in the combined custom image. Value range: /dev/xvda to /dev/xvdz.
      * `disk_type` (`str`) - Specifies the type of a disk in the combined custom image. If you specify this parameter, you can use a data disk snapshot as the data source of a system disk for creating an image. If it is not specified, the disk type is determined by the corresponding snapshot. Valid values: `system`, `data`,
      * `size` (`float`) - Specifies the size of a disk in the combined custom image, in GiB. Value range: 5 to 2000.
      * `snapshot_id` (`str`) - Specifies a snapshot that is used to create a combined custom image.
    """
    force: pulumi.Output[bool]
    """
    Indicates whether to force delete the custom image, Default is `false`. 
    - true：Force deletes the custom image, regardless of whether the image is currently being used by other instances.
    - false：Verifies that the image is not currently in use by any other instances before deleting the image.
    """
    image_name: pulumi.Output[str]
    """
    The image name. It must be 2 to 128 characters in length, and must begin with a letter or Chinese character (beginning with http:// or https:// is not allowed). It can contain digits, colons (:), underscores (_), or hyphens (-). Default value: null.
    """
    instance_id: pulumi.Output[str]
    """
    The instance ID.
    """
    name: pulumi.Output[str]
    platform: pulumi.Output[str]
    """
    Specifies the operating system platform of the system disk after you specify a data disk snapshot as the data source of the system disk for creating an image. Valid values: `CentOS`, `Ubuntu`, `SUSE`, `OpenSUSE`, `RedHat`, `Debian`, `CoreOS`, `Aliyun Linux`, `Windows Server 2003`, `Windows Server 2008`, `Windows Server 2012`, `Windows 7`, Default is `Others Linux`, `Customized Linux`.
    """
    resource_group_id: pulumi.Output[str]
    """
    The ID of the enterprise resource group to which a custom image belongs
    """
    snapshot_id: pulumi.Output[str]
    """
    Specifies a snapshot that is used to create a combined custom image.
    """
    tags: pulumi.Output[dict]
    """
    The tag value of an image. The value of N ranges from 1 to 20.
    """
    def __init__(__self__, resource_name, opts=None, architecture=None, description=None, disk_device_mappings=None, force=None, image_name=None, instance_id=None, name=None, platform=None, resource_group_id=None, snapshot_id=None, tags=None, __props__=None, __name__=None, __opts__=None):
        """
        Creates a custom image. You can then use a custom image to create ECS instances (RunInstances) or change the system disk for an existing instance (ReplaceSystemDisk).
        
        > **NOTE:**  If you want to create a template from an ECS instance, you can specify the instance ID (InstanceId) to create a custom image. You must make sure that the status of the specified instance is Running or Stopped. After a successful invocation, each disk of the specified instance has a new snapshot created.
        
        > **NOTE:**  If you want to create a custom image based on the system disk of your ECS instance, you can specify one of the system disk snapshots (SnapshotId) to create a custom image. However, the specified snapshot cannot be created on or before July 15, 2013.
        
        > **NOTE:**  If you want to combine snapshots of multiple disks into an image template, you can specify DiskDeviceMapping to create a custom image.
        
        > **NOTE:**  Available in 1.64.0+
        
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] architecture: Specifies the architecture of the system disk after you specify a data disk snapshot as the data source of the system disk for creating an image. Valid values: `i386` , Default is `x86_64`.
        :param pulumi.Input[str] description: The description of the image. It must be 2 to 256 characters in length and must not start with http:// or https://. Default value: null.
        :param pulumi.Input[list] disk_device_mappings: Description of the system with disks and snapshots under the image.
        :param pulumi.Input[bool] force: Indicates whether to force delete the custom image, Default is `false`. 
               - true：Force deletes the custom image, regardless of whether the image is currently being used by other instances.
               - false：Verifies that the image is not currently in use by any other instances before deleting the image.
        :param pulumi.Input[str] image_name: The image name. It must be 2 to 128 characters in length, and must begin with a letter or Chinese character (beginning with http:// or https:// is not allowed). It can contain digits, colons (:), underscores (_), or hyphens (-). Default value: null.
        :param pulumi.Input[str] instance_id: The instance ID.
        :param pulumi.Input[str] platform: Specifies the operating system platform of the system disk after you specify a data disk snapshot as the data source of the system disk for creating an image. Valid values: `CentOS`, `Ubuntu`, `SUSE`, `OpenSUSE`, `RedHat`, `Debian`, `CoreOS`, `Aliyun Linux`, `Windows Server 2003`, `Windows Server 2008`, `Windows Server 2012`, `Windows 7`, Default is `Others Linux`, `Customized Linux`.
        :param pulumi.Input[str] resource_group_id: The ID of the enterprise resource group to which a custom image belongs
        :param pulumi.Input[str] snapshot_id: Specifies a snapshot that is used to create a combined custom image.
        :param pulumi.Input[dict] tags: The tag value of an image. The value of N ranges from 1 to 20.
        
        The **disk_device_mappings** object supports the following:
        
          * `device` (`pulumi.Input[str]`) - Specifies the name of a disk in the combined custom image. Value range: /dev/xvda to /dev/xvdz.
          * `disk_type` (`pulumi.Input[str]`) - Specifies the type of a disk in the combined custom image. If you specify this parameter, you can use a data disk snapshot as the data source of a system disk for creating an image. If it is not specified, the disk type is determined by the corresponding snapshot. Valid values: `system`, `data`,
          * `size` (`pulumi.Input[float]`) - Specifies the size of a disk in the combined custom image, in GiB. Value range: 5 to 2000.
          * `snapshot_id` (`pulumi.Input[str]`) - Specifies a snapshot that is used to create a combined custom image.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-alicloud/blob/master/website/docs/r/image.html.markdown.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['architecture'] = architecture
            __props__['description'] = description
            __props__['disk_device_mappings'] = disk_device_mappings
            __props__['force'] = force
            __props__['image_name'] = image_name
            __props__['instance_id'] = instance_id
            __props__['name'] = name
            __props__['platform'] = platform
            __props__['resource_group_id'] = resource_group_id
            __props__['snapshot_id'] = snapshot_id
            __props__['tags'] = tags
        super(Image, __self__).__init__(
            'alicloud:ecs/image:Image',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, architecture=None, description=None, disk_device_mappings=None, force=None, image_name=None, instance_id=None, name=None, platform=None, resource_group_id=None, snapshot_id=None, tags=None):
        """
        Get an existing Image resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.
        
        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] architecture: Specifies the architecture of the system disk after you specify a data disk snapshot as the data source of the system disk for creating an image. Valid values: `i386` , Default is `x86_64`.
        :param pulumi.Input[str] description: The description of the image. It must be 2 to 256 characters in length and must not start with http:// or https://. Default value: null.
        :param pulumi.Input[list] disk_device_mappings: Description of the system with disks and snapshots under the image.
        :param pulumi.Input[bool] force: Indicates whether to force delete the custom image, Default is `false`. 
               - true：Force deletes the custom image, regardless of whether the image is currently being used by other instances.
               - false：Verifies that the image is not currently in use by any other instances before deleting the image.
        :param pulumi.Input[str] image_name: The image name. It must be 2 to 128 characters in length, and must begin with a letter or Chinese character (beginning with http:// or https:// is not allowed). It can contain digits, colons (:), underscores (_), or hyphens (-). Default value: null.
        :param pulumi.Input[str] instance_id: The instance ID.
        :param pulumi.Input[str] platform: Specifies the operating system platform of the system disk after you specify a data disk snapshot as the data source of the system disk for creating an image. Valid values: `CentOS`, `Ubuntu`, `SUSE`, `OpenSUSE`, `RedHat`, `Debian`, `CoreOS`, `Aliyun Linux`, `Windows Server 2003`, `Windows Server 2008`, `Windows Server 2012`, `Windows 7`, Default is `Others Linux`, `Customized Linux`.
        :param pulumi.Input[str] resource_group_id: The ID of the enterprise resource group to which a custom image belongs
        :param pulumi.Input[str] snapshot_id: Specifies a snapshot that is used to create a combined custom image.
        :param pulumi.Input[dict] tags: The tag value of an image. The value of N ranges from 1 to 20.
        
        The **disk_device_mappings** object supports the following:
        
          * `device` (`pulumi.Input[str]`) - Specifies the name of a disk in the combined custom image. Value range: /dev/xvda to /dev/xvdz.
          * `disk_type` (`pulumi.Input[str]`) - Specifies the type of a disk in the combined custom image. If you specify this parameter, you can use a data disk snapshot as the data source of a system disk for creating an image. If it is not specified, the disk type is determined by the corresponding snapshot. Valid values: `system`, `data`,
          * `size` (`pulumi.Input[float]`) - Specifies the size of a disk in the combined custom image, in GiB. Value range: 5 to 2000.
          * `snapshot_id` (`pulumi.Input[str]`) - Specifies a snapshot that is used to create a combined custom image.

        > This content is derived from https://github.com/terraform-providers/terraform-provider-alicloud/blob/master/website/docs/r/image.html.markdown.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()
        __props__["architecture"] = architecture
        __props__["description"] = description
        __props__["disk_device_mappings"] = disk_device_mappings
        __props__["force"] = force
        __props__["image_name"] = image_name
        __props__["instance_id"] = instance_id
        __props__["name"] = name
        __props__["platform"] = platform
        __props__["resource_group_id"] = resource_group_id
        __props__["snapshot_id"] = snapshot_id
        __props__["tags"] = tags
        return Image(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

_SNAKE_TO_CAMEL_CASE_TABLE = {
    "access_group_name": "accessGroupName",
    "access_key": "accessKey",
    "access_point_id": "accessPointId",
    "access_rule_id": "accessRuleId",
    "accessed_by": "accessedBy",
    "account_alias": "accountAlias",
    "account_description": "accountDescription",
    "account_id": "accountId",
    "account_name": "accountName",
    "account_password": "accountPassword",
    "account_privilege": "accountPrivilege",
    "account_type": "accountType",
    "acl_id": "aclId",
    "acl_operation_type": "aclOperationType",
    "acl_resource_name": "aclResourceName",
    "acl_resource_pattern_type": "aclResourcePatternType",
    "acl_resource_type": "aclResourceType",
    "acl_status": "aclStatus",
    "acl_type": "aclType",
    "address_ip_version": "addressIpVersion",
    "address_type": "addressType",
    "adjustment_type": "adjustmentType",
    "adjustment_value": "adjustmentValue",
    "agent_version": "agentVersion",
    "alarm_actions": "alarmActions",
    "alicloud_certifacte_id": "alicloudCertifacteId",
    "alicloud_certifacte_name": "alicloudCertifacteName",
    "alicloud_certificate_id": "alicloudCertificateId",
    "alicloud_certificate_name": "alicloudCertificateName",
    "alicloud_certificate_region_id": "alicloudCertificateRegionId",
    "allocate_public_ip": "allocatePublicIp",
    "allocation_id": "allocationId",
    "api_id": "apiId",
    "api_server_internet": "apiServerInternet",
    "api_server_intranet": "apiServerIntranet",
    "app_id": "appId",
    "append_meta": "appendMeta",
    "archive_backup_keep_count": "archiveBackupKeepCount",
    "archive_backup_keep_policy": "archiveBackupKeepPolicy",
    "archive_backup_retention_period": "archiveBackupRetentionPeriod",
    "assume_role": "assumeRole",
    "attachment_count": "attachmentCount",
    "auth_config": "authConfig",
    "auth_type": "authType",
    "auto_create": "autoCreate",
    "auto_release_time": "autoReleaseTime",
    "auto_renew": "autoRenew",
    "auto_renew_period": "autoRenewPeriod",
    "auto_split": "autoSplit",
    "auto_upgrade_minor_version": "autoUpgradeMinorVersion",
    "availability_zone": "availabilityZone",
    "backend_port": "backendPort",
    "backend_servers": "backendServers",
    "backup_id": "backupId",
    "backup_periods": "backupPeriods",
    "backup_retention_period": "backupRetentionPeriod",
    "backup_time": "backupTime",
    "bandwidth_limit": "bandwidthLimit",
    "bandwidth_package_id": "bandwidthPackageId",
    "bandwidth_package_ids": "bandwidthPackageIds",
    "bandwidth_packages": "bandwidthPackages",
    "base_bandwidth": "baseBandwidth",
    "block_ips": "blockIps",
    "blue_green": "blueGreen",
    "blue_green_confirm": "blueGreenConfirm",
    "business_status": "businessStatus",
    "ca_cert": "caCert",
    "ca_certificate": "caCertificate",
    "cache_configs": "cacheConfigs",
    "cache_control": "cacheControl",
    "ccn_id": "ccnId",
    "cdn_type": "cdnType",
    "cen_id": "cenId",
    "cen_owner_id": "cenOwnerId",
    "cen_uid": "cenUid",
    "certificate_config": "certificateConfig",
    "character_set": "characterSet",
    "character_set_name": "characterSetName",
    "charge_type": "chargeType",
    "child_instance_id": "childInstanceId",
    "child_instance_owner_id": "childInstanceOwnerId",
    "child_instance_region_id": "childInstanceRegionId",
    "cidr_block": "cidrBlock",
    "cidr_ip": "cidrIp",
    "ciphertext_blob": "ciphertextBlob",
    "client_cert": "clientCert",
    "client_config": "clientConfig",
    "client_ip": "clientIp",
    "client_ip_pool": "clientIpPool",
    "client_key": "clientKey",
    "cloud_monitor_group_id": "cloudMonitorGroupId",
    "cluster_ca_cert": "clusterCaCert",
    "cluster_id": "clusterId",
    "cluster_name": "clusterName",
    "cluster_network_type": "clusterNetworkType",
    "cluster_type": "clusterType",
    "code_checksum": "codeChecksum",
    "compact_topic": "compactTopic",
    "comparison_operator": "comparisonOperator",
    "compress_type": "compressType",
    "config_mns": "configMns",
    "configuration_source": "configurationSource",
    "connection_domain": "connectionDomain",
    "connection_prefix": "connectionPrefix",
    "connection_string": "connectionString",
    "constant_parameters": "constantParameters",
    "consumer_id": "consumerId",
    "contact_groups": "contactGroups",
    "content_disposition": "contentDisposition",
    "content_encoding": "contentEncoding",
    "content_length": "contentLength",
    "content_md5": "contentMd5",
    "content_type": "contentType",
    "cookie_timeout": "cookieTimeout",
    "cool_down_duration": "coolDownDuration",
    "cors_rules": "corsRules",
    "coupon_id": "couponId",
    "create_time": "createTime",
    "creation_date": "creationDate",
    "creation_time": "creationTime",
    "credit_specification": "creditSpecification",
    "customer_gateway_id": "customerGatewayId",
    "data_disks": "dataDisks",
    "data_node_amount": "dataNodeAmount",
    "data_node_disk_size": "dataNodeDiskSize",
    "data_node_disk_type": "dataNodeDiskType",
    "data_node_spec": "dataNodeSpec",
    "db_cluster_id": "dbClusterId",
    "db_description": "dbDescription",
    "db_endpoint_id": "dbEndpointId",
    "db_instance_class": "dbInstanceClass",
    "db_instance_ids": "dbInstanceIds",
    "db_instance_storage": "dbInstanceStorage",
    "db_instance_storage_type": "dbInstanceStorageType",
    "db_name": "dbName",
    "db_names": "dbNames",
    "db_node_class": "dbNodeClass",
    "db_type": "dbType",
    "db_version": "dbVersion",
    "default_cooldown": "defaultCooldown",
    "default_domain": "defaultDomain",
    "default_result": "defaultResult",
    "default_visibility": "defaultVisibility",
    "defer_scale_in_duration": "deferScaleInDuration",
    "delay_seconds": "delaySeconds",
    "delete_auto_snapshot": "deleteAutoSnapshot",
    "delete_protection": "deleteProtection",
    "delete_protection_validation": "deleteProtectionValidation",
    "delete_with_instance": "deleteWithInstance",
    "deletion_protection": "deletionProtection",
    "deletion_window_in_days": "deletionWindowInDays",
    "deploy_type": "deployType",
    "deposit_type": "depositType",
    "dest_cidr": "destCidr",
    "dest_port_range": "destPortRange",
    "destination_cidrblock": "destinationCidrblock",
    "deviation_cell_version_in_sec": "deviationCellVersionInSec",
    "device_name": "deviceName",
    "disable_scale_in": "disableScaleIn",
    "disk_category": "diskCategory",
    "disk_device_mappings": "diskDeviceMappings",
    "disk_id": "diskId",
    "disk_size": "diskSize",
    "disk_type": "diskType",
    "display_name": "displayName",
    "distribution_type": "distributionType",
    "dns_servers": "dnsServers",
    "domain_count": "domainCount",
    "domain_id": "domainId",
    "domain_list": "domainList",
    "domain_name": "domainName",
    "dry_run": "dryRun",
    "eas_enable": "easEnable",
    "ecs_role_name": "ecsRoleName",
    "effect_immediately": "effectImmediately",
    "effective_interval": "effectiveInterval",
    "eip_max": "eipMax",
    "emr_ver": "emrVer",
    "enable_auto_snapshot": "enableAutoSnapshot",
    "enable_backup_log": "enableBackupLog",
    "enable_http2": "enableHttp2",
    "enable_ipsec": "enableIpsec",
    "enable_ssh": "enableSsh",
    "enable_ssl": "enableSsl",
    "enable_web_tracking": "enableWebTracking",
    "encrypted_secret": "encryptedSecret",
    "encryption_context": "encryptionContext",
    "end_time": "endTime",
    "endpoint_public_access_enabled": "endpointPublicAccessEnabled",
    "engine_version": "engineVersion",
    "entry_lists": "entryLists",
    "environment_variables": "environmentVariables",
    "established_timeout": "establishedTimeout",
    "estimated_instance_warmup": "estimatedInstanceWarmup",
    "evaluation_count": "evaluationCount",
    "event_rw": "eventRw",
    "expired_time": "expiredTime",
    "external_ip": "externalIp",
    "external_port": "externalPort",
    "extranet_endpoint": "extranetEndpoint",
    "fc_service_config": "fcServiceConfig",
    "field_searches": "fieldSearches",
    "file_system_id": "fileSystemId",
    "filter_tag": "filterTag",
    "finger_print": "fingerPrint",
    "force_delete": "forceDelete",
    "force_destroy": "forceDestroy",
    "force_update": "forceUpdate",
    "forward_entry_id": "forwardEntryId",
    "forward_port": "forwardPort",
    "forward_table_id": "forwardTableId",
    "forward_table_ids": "forwardTableIds",
    "frontend_port": "frontendPort",
    "full_text": "fullText",
    "function_args": "functionArgs",
    "function_id": "functionId",
    "function_name": "functionName",
    "geographic_region_ids": "geographicRegionIds",
    "group_id": "groupId",
    "group_name": "groupName",
    "hard_expiry": "hardExpiry",
    "havip_id": "havipId",
    "health_check": "healthCheck",
    "health_check_connect_port": "healthCheckConnectPort",
    "health_check_domain": "healthCheckDomain",
    "health_check_http_code": "healthCheckHttpCode",
    "health_check_interval": "healthCheckInterval",
    "health_check_method": "healthCheckMethod",
    "health_check_source_ip": "healthCheckSourceIp",
    "health_check_target_ip": "healthCheckTargetIp",
    "health_check_timeout": "healthCheckTimeout",
    "health_check_type": "healthCheckType",
    "health_check_uri": "healthCheckUri",
    "healthy_threshold": "healthyThreshold",
    "heartbeat_timeout": "heartbeatTimeout",
    "high_availability_enable": "highAvailabilityEnable",
    "high_space_usage_protection": "highSpaceUsageProtection",
    "host_groups": "hostGroups",
    "host_name": "hostName",
    "host_record": "hostRecord",
    "http_header_configs": "httpHeaderConfigs",
    "http_service_config": "httpServiceConfig",
    "http_vpc_service_config": "httpVpcServiceConfig",
    "identify_lists": "identifyLists",
    "identify_type": "identifyType",
    "idle_timeout": "idleTimeout",
    "ike_configs": "ikeConfigs",
    "image_id": "imageId",
    "image_name": "imageName",
    "image_owner_alias": "imageOwnerAlias",
    "include_data_disks": "includeDataDisks",
    "inner_access": "innerAccess",
    "inner_access_policy": "innerAccessPolicy",
    "input_detail": "inputDetail",
    "input_type": "inputType",
    "install_cloud_monitor": "installCloudMonitor",
    "instance_amount": "instanceAmount",
    "instance_charge_type": "instanceChargeType",
    "instance_class": "instanceClass",
    "instance_group_count": "instanceGroupCount",
    "instance_id": "instanceId",
    "instance_ids": "instanceIds",
    "instance_name": "instanceName",
    "instance_port": "instancePort",
    "instance_series": "instanceSeries",
    "instance_status": "instanceStatus",
    "instance_storage": "instanceStorage",
    "instance_type": "instanceType",
    "instance_types": "instanceTypes",
    "interface_id": "interfaceId",
    "internal_ip": "internalIp",
    "internal_port": "internalPort",
    "internet_access": "internetAccess",
    "internet_charge_type": "internetChargeType",
    "internet_ip": "internetIp",
    "internet_max_bandwidth_in": "internetMaxBandwidthIn",
    "internet_max_bandwidth_out": "internetMaxBandwidthOut",
    "intranet_endpoint": "intranetEndpoint",
    "io_max": "ioMax",
    "io_optimized": "ioOptimized",
    "ip_address": "ipAddress",
    "ip_count": "ipCount",
    "ip_protocol": "ipProtocol",
    "ip_type": "ipType",
    "ip_version": "ipVersion",
    "ipsec_configs": "ipsecConfigs",
    "is_default": "isDefault",
    "is_enabled": "isEnabled",
    "is_open_public_ip": "isOpenPublicIp",
    "is_outdated": "isOutdated",
    "is_ptr": "isPtr",
    "key_file": "keyFile",
    "key_fingerprint": "keyFingerprint",
    "key_id": "keyId",
    "key_name": "keyName",
    "key_name_prefix": "keyNamePrefix",
    "key_pair_name": "keyPairName",
    "key_usage": "keyUsage",
    "kibana_domain": "kibanaDomain",
    "kibana_port": "kibanaPort",
    "kibana_whitelists": "kibanaWhitelists",
    "kms_encrypted_password": "kmsEncryptedPassword",
    "kms_encryption_context": "kmsEncryptionContext",
    "kms_key_id": "kmsKeyId",
    "kube_config": "kubeConfig",
    "last_modified": "lastModified",
    "last_modify_time": "lastModifyTime",
    "latest_image": "latestImage",
    "launch_expiration_time": "launchExpirationTime",
    "launch_time": "launchTime",
    "lb_port": "lbPort",
    "lb_protocol": "lbProtocol",
    "license_code": "licenseCode",
    "license_type": "licenseType",
    "life_cycle": "lifeCycle",
    "lifecycle_rules": "lifecycleRules",
    "lifecycle_transition": "lifecycleTransition",
    "limit_type": "limitType",
    "listener_forward": "listenerForward",
    "listener_sync": "listenerSync",
    "load_balancer_id": "loadBalancerId",
    "loadbalancer_ids": "loadbalancerIds",
    "local_log_retention_hours": "localLogRetentionHours",
    "local_log_retention_space": "localLogRetentionSpace",
    "local_subnet": "localSubnet",
    "local_subnets": "localSubnets",
    "local_topic": "localTopic",
    "log_backup": "logBackup",
    "log_backup_frequency": "logBackupFrequency",
    "log_backup_retention_period": "logBackupRetentionPeriod",
    "log_config": "logConfig",
    "log_endpoint": "logEndpoint",
    "log_retention_period": "logRetentionPeriod",
    "log_sample": "logSample",
    "logging_enabled": "loggingEnabled",
    "logging_isenable": "loggingIsenable",
    "logtail_config_name": "logtailConfigName",
    "machine_group_name": "machineGroupName",
    "maintain_end_time": "maintainEndTime",
    "maintain_start_time": "maintainStartTime",
    "maintain_time": "maintainTime",
    "master_auto_renew": "masterAutoRenew",
    "master_auto_renew_period": "masterAutoRenewPeriod",
    "master_db_instance_id": "masterDbInstanceId",
    "master_disk_category": "masterDiskCategory",
    "master_disk_size": "masterDiskSize",
    "master_instance_charge_type": "masterInstanceChargeType",
    "master_instance_type": "masterInstanceType",
    "master_instance_types": "masterInstanceTypes",
    "master_node_spec": "masterNodeSpec",
    "master_nodes": "masterNodes",
    "master_period": "masterPeriod",
    "master_period_unit": "masterPeriodUnit",
    "master_public_ip": "masterPublicIp",
    "master_pwd": "masterPwd",
    "master_slave_server_group_id": "masterSlaveServerGroupId",
    "master_zone_id": "masterZoneId",
    "max_bandwidth_abs": "maxBandwidthAbs",
    "max_bandwidth_percent": "maxBandwidthPercent",
    "max_connections": "maxConnections",
    "max_delay_time": "maxDelayTime",
    "max_login_attempts": "maxLoginAttempts",
    "max_password_age": "maxPasswordAge",
    "max_size": "maxSize",
    "max_split_shard_count": "maxSplitShardCount",
    "max_version": "maxVersion",
    "maximum_message_size": "maximumMessageSize",
    "memory_size": "memorySize",
    "message_retention_period": "messageRetentionPeriod",
    "message_type": "messageType",
    "metric_name": "metricName",
    "metric_type": "metricType",
    "mfa_bind_required": "mfaBindRequired",
    "min_bandwidth_abs": "minBandwidthAbs",
    "min_bandwidth_percent": "minBandwidthPercent",
    "min_size": "minSize",
    "minimum_password_length": "minimumPasswordLength",
    "mns_endpoint": "mnsEndpoint",
    "mock_service_config": "mockServiceConfig",
    "mongo_lists": "mongoLists",
    "monitoring_period": "monitoringPeriod",
    "multi_az_policy": "multiAzPolicy",
    "name_prefix": "namePrefix",
    "nat_gateway_id": "natGatewayId",
    "need_slb": "needSlb",
    "net_type": "netType",
    "network_acl_id": "networkAclId",
    "network_interface_id": "networkInterfaceId",
    "network_interfaces": "networkInterfaces",
    "network_type": "networkType",
    "new_nat_gateway": "newNatGateway",
    "next_hop": "nextHop",
    "nexthop_id": "nexthopId",
    "nexthop_type": "nexthopType",
    "nic_type": "nicType",
    "node_cidr_mask": "nodeCidrMask",
    "node_number": "nodeNumber",
    "notification_arn": "notificationArn",
    "notification_metadata": "notificationMetadata",
    "notification_types": "notificationTypes",
    "notify_content_format": "notifyContentFormat",
    "notify_strategy": "notifyStrategy",
    "offering_type": "offeringType",
    "on_demand_base_capacity": "onDemandBaseCapacity",
    "on_demand_percentage_above_base_capacity": "onDemandPercentageAboveBaseCapacity",
    "opposite_access_point_id": "oppositeAccessPointId",
    "opposite_interface_id": "oppositeInterfaceId",
    "opposite_interface_owner_id": "oppositeInterfaceOwnerId",
    "opposite_region": "oppositeRegion",
    "opposite_router_id": "oppositeRouterId",
    "opposite_router_type": "oppositeRouterType",
    "optimize_enable": "optimizeEnable",
    "option_software_lists": "optionSoftwareLists",
    "os_type": "osType",
    "oss_bucket": "ossBucket",
    "oss_bucket_name": "ossBucketName",
    "oss_key": "ossKey",
    "oss_key_prefix": "ossKeyPrefix",
    "oss_prefix": "ossPrefix",
    "ots_instance_name": "otsInstanceName",
    "output_type": "outputType",
    "package_version": "packageVersion",
    "page404_config": "page404Config",
    "page_compress_enable": "pageCompressEnable",
    "paid_type": "paidType",
    "parameter_filter_config": "parameterFilterConfig",
    "partition_num": "partitionNum",
    "password_inherit": "passwordInherit",
    "password_reset_required": "passwordResetRequired",
    "password_reuse_prevention": "passwordReusePrevention",
    "pay_type": "payType",
    "percent_source_type": "percentSourceType",
    "period_unit": "periodUnit",
    "persistence_timeout": "persistenceTimeout",
    "pgp_key": "pgpKey",
    "plan_code": "planCode",
    "pod_cidr": "podCidr",
    "policy_name": "policyName",
    "policy_type": "policyType",
    "polling_wait_seconds": "pollingWaitSeconds",
    "port_count": "portCount",
    "port_range": "portRange",
    "preferred_backup_periods": "preferredBackupPeriods",
    "preferred_backup_time": "preferredBackupTime",
    "pricing_cycle": "pricingCycle",
    "primary_keys": "primaryKeys",
    "private_ip": "privateIp",
    "private_ip_address": "privateIpAddress",
    "private_ips": "privateIps",
    "private_ips_count": "privateIpsCount",
    "private_key": "privateKey",
    "private_whitelists": "privateWhitelists",
    "private_zone": "privateZone",
    "product_code": "productCode",
    "project_name": "projectName",
    "protocol_type": "protocolType",
    "proxy_pattern": "proxyPattern",
    "public_ip": "publicIp",
    "public_key": "publicKey",
    "public_whitelists": "publicWhitelists",
    "publish_vpc": "publishVpc",
    "qos_id": "qosId",
    "ram_role_name": "ramRoleName",
    "ram_users": "ramUsers",
    "range_enable": "rangeEnable",
    "read_enable": "readEnable",
    "record_count": "recordCount",
    "record_id": "recordId",
    "record_schema": "recordSchema",
    "record_type": "recordType",
    "recurrence_end_time": "recurrenceEndTime",
    "recurrence_type": "recurrenceType",
    "recurrence_value": "recurrenceValue",
    "refer_config": "referConfig",
    "referer_config": "refererConfig",
    "region_ids": "regionIds",
    "related_cluster_id": "relatedClusterId",
    "release_eip": "releaseEip",
    "release_time": "releaseTime",
    "remote_subnets": "remoteSubnets",
    "removal_policies": "removalPolicies",
    "renewal_status": "renewalStatus",
    "repeat_weekdays": "repeatWeekdays",
    "replica_set_name": "replicaSetName",
    "replication_factor": "replicationFactor",
    "repo_type": "repoType",
    "request_config": "requestConfig",
    "request_parameters": "requestParameters",
    "request_timeout": "requestTimeout",
    "require_lowercase_characters": "requireLowercaseCharacters",
    "require_numbers": "requireNumbers",
    "require_symbols": "requireSymbols",
    "require_uppercase_characters": "requireUppercaseCharacters",
    "resource_group_id": "resourceGroupId",
    "resource_record": "resourceRecord",
    "retention_days": "retentionDays",
    "retention_period": "retentionPeriod",
    "role_id": "roleId",
    "role_name": "roleName",
    "route_dest": "routeDest",
    "route_table_id": "routeTableId",
    "router_id": "routerId",
    "router_table_id": "routerTableId",
    "router_type": "routerType",
    "rw_access_type": "rwAccessType",
    "sag_id": "sagId",
    "scaling_configuration_name": "scalingConfigurationName",
    "scaling_group_id": "scalingGroupId",
    "scaling_group_name": "scalingGroupName",
    "scaling_rule_name": "scalingRuleName",
    "scaling_rule_type": "scalingRuleType",
    "scheduled_action": "scheduledAction",
    "scheduled_task_name": "scheduledTaskName",
    "secret_file": "secretFile",
    "secret_key": "secretKey",
    "security_enhancement_strategy": "securityEnhancementStrategy",
    "security_group_id": "securityGroupId",
    "security_group_ids": "securityGroupIds",
    "security_group_type": "securityGroupType",
    "security_groups": "securityGroups",
    "security_ip_lists": "securityIpLists",
    "security_ip_mode": "securityIpMode",
    "security_ips": "securityIps",
    "security_token": "securityToken",
    "server_certificate": "serverCertificate",
    "server_certificate_id": "serverCertificateId",
    "server_group_id": "serverGroupId",
    "server_side_encryption": "serverSideEncryption",
    "server_side_encryption_rule": "serverSideEncryptionRule",
    "server_type": "serverType",
    "service_bandwidth": "serviceBandwidth",
    "service_cidr": "serviceCidr",
    "service_domain": "serviceDomain",
    "service_id": "serviceId",
    "service_type": "serviceType",
    "shard_count": "shardCount",
    "shard_lists": "shardLists",
    "shared_credentials_file": "sharedCredentialsFile",
    "silence_time": "silenceTime",
    "skip_region_validation": "skipRegionValidation",
    "slave_zone_id": "slaveZoneId",
    "slb_id": "slbId",
    "slb_internet": "slbInternet",
    "slb_internet_enabled": "slbInternetEnabled",
    "slb_intranet": "slbIntranet",
    "sls_project_arn": "slsProjectArn",
    "sls_write_role_arn": "slsWriteRoleArn",
    "snapshot_id": "snapshotId",
    "snat_entry_id": "snatEntryId",
    "snat_ip": "snatIp",
    "snat_table_id": "snatTableId",
    "snat_table_ids": "snatTableIds",
    "source_arn": "sourceArn",
    "source_cidr": "sourceCidr",
    "source_cidr_ip": "sourceCidrIp",
    "source_group_owner_account": "sourceGroupOwnerAccount",
    "source_image_id": "sourceImageId",
    "source_port": "sourcePort",
    "source_port_range": "sourcePortRange",
    "source_region_id": "sourceRegionId",
    "source_security_group_id": "sourceSecurityGroupId",
    "source_type": "sourceType",
    "source_vswitch_id": "sourceVswitchId",
    "spec_type": "specType",
    "spot_instance_pools": "spotInstancePools",
    "spot_instance_remedy": "spotInstanceRemedy",
    "spot_price_limit": "spotPriceLimit",
    "spot_strategy": "spotStrategy",
    "sql_collector_config_value": "sqlCollectorConfigValue",
    "sql_collector_status": "sqlCollectorStatus",
    "ssh_enable": "sshEnable",
    "ssl_certificate_id": "sslCertificateId",
    "ssl_connections": "sslConnections",
    "ssl_vpn_server_id": "sslVpnServerId",
    "stage_name": "stageName",
    "stage_names": "stageNames",
    "start_time": "startTime",
    "step_adjustments": "stepAdjustments",
    "sticky_session": "stickySession",
    "sticky_session_type": "stickySessionType",
    "storage_class": "storageClass",
    "storage_engine": "storageEngine",
    "storage_type": "storageType",
    "sub_domain": "subDomain",
    "sub_id": "subId",
    "subnet_id": "subnetId",
    "system_disk_category": "systemDiskCategory",
    "system_disk_description": "systemDiskDescription",
    "system_disk_name": "systemDiskName",
    "system_disk_size": "systemDiskSize",
    "system_parameters": "systemParameters",
    "table_name": "tableName",
    "target_value": "targetValue",
    "task_enabled": "taskEnabled",
    "time_points": "timePoints",
    "time_to_live": "timeToLive",
    "tls_cipher_policy": "tlsCipherPolicy",
    "topic_name": "topicName",
    "topic_quota": "topicQuota",
    "trigger_id": "triggerId",
    "triggered_count": "triggeredCount",
    "unhealthy_threshold": "unhealthyThreshold",
    "update_time": "updateTime",
    "use_local_metadb": "useLocalMetadb",
    "user_access_type": "userAccessType",
    "user_ca": "userCa",
    "user_client_ip": "userClientIp",
    "user_data": "userData",
    "user_defined_emr_ecs_role": "userDefinedEmrEcsRole",
    "user_mail": "userMail",
    "user_name": "userName",
    "user_names": "userNames",
    "version_id": "versionId",
    "video_seek_enable": "videoSeekEnable",
    "visibility_timeout": "visibilityTimeout",
    "volume_tags": "volumeTags",
    "vpc_auth_mode": "vpcAuthMode",
    "vpc_config": "vpcConfig",
    "vpc_domain": "vpcDomain",
    "vpc_id": "vpcId",
    "vpc_ids": "vpcIds",
    "vpc_name": "vpcName",
    "vpn_gateway_id": "vpnGatewayId",
    "vserver_groups": "vserverGroups",
    "vswitch_id": "vswitchId",
    "vswitch_ids": "vswitchIds",
    "worker_auto_renew": "workerAutoRenew",
    "worker_auto_renew_period": "workerAutoRenewPeriod",
    "worker_data_disk_category": "workerDataDiskCategory",
    "worker_data_disk_size": "workerDataDiskSize",
    "worker_disk_category": "workerDiskCategory",
    "worker_disk_size": "workerDiskSize",
    "worker_instance_charge_type": "workerInstanceChargeType",
    "worker_instance_type": "workerInstanceType",
    "worker_instance_types": "workerInstanceTypes",
    "worker_nodes": "workerNodes",
    "worker_number": "workerNumber",
    "worker_numbers": "workerNumbers",
    "worker_period": "workerPeriod",
    "worker_period_unit": "workerPeriodUnit",
    "x_forwarded_for": "xForwardedFor",
    "zone_count": "zoneCount",
    "zone_id": "zoneId",
}

_CAMEL_TO_SNAKE_CASE_TABLE = {
    "accessGroupName": "access_group_name",
    "accessKey": "access_key",
    "accessPointId": "access_point_id",
    "accessRuleId": "access_rule_id",
    "accessedBy": "accessed_by",
    "accountAlias": "account_alias",
    "accountDescription": "account_description",
    "accountId": "account_id",
    "accountName": "account_name",
    "accountPassword": "account_password",
    "accountPrivilege": "account_privilege",
    "accountType": "account_type",
    "aclId": "acl_id",
    "aclOperationType": "acl_operation_type",
    "aclResourceName": "acl_resource_name",
    "aclResourcePatternType": "acl_resource_pattern_type",
    "aclResourceType": "acl_resource_type",
    "aclStatus": "acl_status",
    "aclType": "acl_type",
    "addressIpVersion": "address_ip_version",
    "addressType": "address_type",
    "adjustmentType": "adjustment_type",
    "adjustmentValue": "adjustment_value",
    "agentVersion": "agent_version",
    "alarmActions": "alarm_actions",
    "alicloudCertifacteId": "alicloud_certifacte_id",
    "alicloudCertifacteName": "alicloud_certifacte_name",
    "alicloudCertificateId": "alicloud_certificate_id",
    "alicloudCertificateName": "alicloud_certificate_name",
    "alicloudCertificateRegionId": "alicloud_certificate_region_id",
    "allocatePublicIp": "allocate_public_ip",
    "allocationId": "allocation_id",
    "apiId": "api_id",
    "apiServerInternet": "api_server_internet",
    "apiServerIntranet": "api_server_intranet",
    "appId": "app_id",
    "appendMeta": "append_meta",
    "archiveBackupKeepCount": "archive_backup_keep_count",
    "archiveBackupKeepPolicy": "archive_backup_keep_policy",
    "archiveBackupRetentionPeriod": "archive_backup_retention_period",
    "assumeRole": "assume_role",
    "attachmentCount": "attachment_count",
    "authConfig": "auth_config",
    "authType": "auth_type",
    "autoCreate": "auto_create",
    "autoReleaseTime": "auto_release_time",
    "autoRenew": "auto_renew",
    "autoRenewPeriod": "auto_renew_period",
    "autoSplit": "auto_split",
    "autoUpgradeMinorVersion": "auto_upgrade_minor_version",
    "availabilityZone": "availability_zone",
    "backendPort": "backend_port",
    "backendServers": "backend_servers",
    "backupId": "backup_id",
    "backupPeriods": "backup_periods",
    "backupRetentionPeriod": "backup_retention_period",
    "backupTime": "backup_time",
    "bandwidthLimit": "bandwidth_limit",
    "bandwidthPackageId": "bandwidth_package_id",
    "bandwidthPackageIds": "bandwidth_package_ids",
    "bandwidthPackages": "bandwidth_packages",
    "baseBandwidth": "base_bandwidth",
    "blockIps": "block_ips",
    "blueGreen": "blue_green",
    "blueGreenConfirm": "blue_green_confirm",
    "businessStatus": "business_status",
    "caCert": "ca_cert",
    "caCertificate": "ca_certificate",
    "cacheConfigs": "cache_configs",
    "cacheControl": "cache_control",
    "ccnId": "ccn_id",
    "cdnType": "cdn_type",
    "cenId": "cen_id",
    "cenOwnerId": "cen_owner_id",
    "cenUid": "cen_uid",
    "certificateConfig": "certificate_config",
    "characterSet": "character_set",
    "characterSetName": "character_set_name",
    "chargeType": "charge_type",
    "childInstanceId": "child_instance_id",
    "childInstanceOwnerId": "child_instance_owner_id",
    "childInstanceRegionId": "child_instance_region_id",
    "cidrBlock": "cidr_block",
    "cidrIp": "cidr_ip",
    "ciphertextBlob": "ciphertext_blob",
    "clientCert": "client_cert",
    "clientConfig": "client_config",
    "clientIp": "client_ip",
    "clientIpPool": "client_ip_pool",
    "clientKey": "client_key",
    "cloudMonitorGroupId": "cloud_monitor_group_id",
    "clusterCaCert": "cluster_ca_cert",
    "clusterId": "cluster_id",
    "clusterName": "cluster_name",
    "clusterNetworkType": "cluster_network_type",
    "clusterType": "cluster_type",
    "codeChecksum": "code_checksum",
    "compactTopic": "compact_topic",
    "comparisonOperator": "comparison_operator",
    "compressType": "compress_type",
    "configMns": "config_mns",
    "configurationSource": "configuration_source",
    "connectionDomain": "connection_domain",
    "connectionPrefix": "connection_prefix",
    "connectionString": "connection_string",
    "constantParameters": "constant_parameters",
    "consumerId": "consumer_id",
    "contactGroups": "contact_groups",
    "contentDisposition": "content_disposition",
    "contentEncoding": "content_encoding",
    "contentLength": "content_length",
    "contentMd5": "content_md5",
    "contentType": "content_type",
    "cookieTimeout": "cookie_timeout",
    "coolDownDuration": "cool_down_duration",
    "corsRules": "cors_rules",
    "couponId": "coupon_id",
    "createTime": "create_time",
    "creationDate": "creation_date",
    "creationTime": "creation_time",
    "creditSpecification": "credit_specification",
    "customerGatewayId": "customer_gateway_id",
    "dataDisks": "data_disks",
    "dataNodeAmount": "data_node_amount",
    "dataNodeDiskSize": "data_node_disk_size",
    "dataNodeDiskType": "data_node_disk_type",
    "dataNodeSpec": "data_node_spec",
    "dbClusterId": "db_cluster_id",
    "dbDescription": "db_description",
    "dbEndpointId": "db_endpoint_id",
    "dbInstanceClass": "db_instance_class",
    "dbInstanceIds": "db_instance_ids",
    "dbInstanceStorage": "db_instance_storage",
    "dbInstanceStorageType": "db_instance_storage_type",
    "dbName": "db_name",
    "dbNames": "db_names",
    "dbNodeClass": "db_node_class",
    "dbType": "db_type",
    "dbVersion": "db_version",
    "defaultCooldown": "default_cooldown",
    "defaultDomain": "default_domain",
    "defaultResult": "default_result",
    "defaultVisibility": "default_visibility",
    "deferScaleInDuration": "defer_scale_in_duration",
    "delaySeconds": "delay_seconds",
    "deleteAutoSnapshot": "delete_auto_snapshot",
    "deleteProtection": "delete_protection",
    "deleteProtectionValidation": "delete_protection_validation",
    "deleteWithInstance": "delete_with_instance",
    "deletionProtection": "deletion_protection",
    "deletionWindowInDays": "deletion_window_in_days",
    "deployType": "deploy_type",
    "depositType": "deposit_type",
    "destCidr": "dest_cidr",
    "destPortRange": "dest_port_range",
    "destinationCidrblock": "destination_cidrblock",
    "deviationCellVersionInSec": "deviation_cell_version_in_sec",
    "deviceName": "device_name",
    "disableScaleIn": "disable_scale_in",
    "diskCategory": "disk_category",
    "diskDeviceMappings": "disk_device_mappings",
    "diskId": "disk_id",
    "diskSize": "disk_size",
    "diskType": "disk_type",
    "displayName": "display_name",
    "distributionType": "distribution_type",
    "dnsServers": "dns_servers",
    "domainCount": "domain_count",
    "domainId": "domain_id",
    "domainList": "domain_list",
    "domainName": "domain_name",
    "dryRun": "dry_run",
    "easEnable": "eas_enable",
    "ecsRoleName": "ecs_role_name",
    "effectImmediately": "effect_immediately",
    "effectiveInterval": "effective_interval",
    "eipMax": "eip_max",
    "emrVer": "emr_ver",
    "enableAutoSnapshot": "enable_auto_snapshot",
    "enableBackupLog": "enable_backup_log",
    "enableHttp2": "enable_http2",
    "enableIpsec": "enable_ipsec",
    "enableSsh": "enable_ssh",
    "enableSsl": "enable_ssl",
    "enableWebTracking": "enable_web_tracking",
    "encryptedSecret": "encrypted_secret",
    "encryptionContext": "encryption_context",
    "endTime": "end_time",
    "endpointPublicAccessEnabled": "endpoint_public_access_enabled",
    "engineVersion": "engine_version",
    "entryLists": "entry_lists",
    "environmentVariables": "environment_variables",
    "establishedTimeout": "established_timeout",
    "estimatedInstanceWarmup": "estimated_instance_warmup",
    "evaluationCount": "evaluation_count",
    "eventRw": "event_rw",
    "expiredTime": "expired_time",
    "externalIp": "external_ip",
    "externalPort": "external_port",
    "extranetEndpoint": "extranet_endpoint",
    "fcServiceConfig": "fc_service_config",
    "fieldSearches": "field_searches",
    "fileSystemId": "file_system_id",
    "filterTag": "filter_tag",
    "fingerPrint": "finger_print",
    "forceDelete": "force_delete",
    "forceDestroy": "force_destroy",
    "forceUpdate": "force_update",
    "forwardEntryId": "forward_entry_id",
    "forwardPort": "forward_port",
    "forwardTableId": "forward_table_id",
    "forwardTableIds": "forward_table_ids",
    "frontendPort": "frontend_port",
    "fullText": "full_text",
    "functionArgs": "function_args",
    "functionId": "function_id",
    "functionName": "function_name",
    "geographicRegionIds": "geographic_region_ids",
    "groupId": "group_id",
    "groupName": "group_name",
    "hardExpiry": "hard_expiry",
    "havipId": "havip_id",
    "healthCheck": "health_check",
    "healthCheckConnectPort": "health_check_connect_port",
    "healthCheckDomain": "health_check_domain",
    "healthCheckHttpCode": "health_check_http_code",
    "healthCheckInterval": "health_check_interval",
    "healthCheckMethod": "health_check_method",
    "healthCheckSourceIp": "health_check_source_ip",
    "healthCheckTargetIp": "health_check_target_ip",
    "healthCheckTimeout": "health_check_timeout",
    "healthCheckType": "health_check_type",
    "healthCheckUri": "health_check_uri",
    "healthyThreshold": "healthy_threshold",
    "heartbeatTimeout": "heartbeat_timeout",
    "highAvailabilityEnable": "high_availability_enable",
    "highSpaceUsageProtection": "high_space_usage_protection",
    "hostGroups": "host_groups",
    "hostName": "host_name",
    "hostRecord": "host_record",
    "httpHeaderConfigs": "http_header_configs",
    "httpServiceConfig": "http_service_config",
    "httpVpcServiceConfig": "http_vpc_service_config",
    "identifyLists": "identify_lists",
    "identifyType": "identify_type",
    "idleTimeout": "idle_timeout",
    "ikeConfigs": "ike_configs",
    "imageId": "image_id",
    "imageName": "image_name",
    "imageOwnerAlias": "image_owner_alias",
    "includeDataDisks": "include_data_disks",
    "innerAccess": "inner_access",
    "innerAccessPolicy": "inner_access_policy",
    "inputDetail": "input_detail",
    "inputType": "input_type",
    "installCloudMonitor": "install_cloud_monitor",
    "instanceAmount": "instance_amount",
    "instanceChargeType": "instance_charge_type",
    "instanceClass": "instance_class",
    "instanceGroupCount": "instance_group_count",
    "instanceId": "instance_id",
    "instanceIds": "instance_ids",
    "instanceName": "instance_name",
    "instancePort": "instance_port",
    "instanceSeries": "instance_series",
    "instanceStatus": "instance_status",
    "instanceStorage": "instance_storage",
    "instanceType": "instance_type",
    "instanceTypes": "instance_types",
    "interfaceId": "interface_id",
    "internalIp": "internal_ip",
    "internalPort": "internal_port",
    "internetAccess": "internet_access",
    "internetChargeType": "internet_charge_type",
    "internetIp": "internet_ip",
    "internetMaxBandwidthIn": "internet_max_bandwidth_in",
    "internetMaxBandwidthOut": "internet_max_bandwidth_out",
    "intranetEndpoint": "intranet_endpoint",
    "ioMax": "io_max",
    "ioOptimized": "io_optimized",
    "ipAddress": "ip_address",
    "ipCount": "ip_count",
    "ipProtocol": "ip_protocol",
    "ipType": "ip_type",
    "ipVersion": "ip_version",
    "ipsecConfigs": "ipsec_configs",
    "isDefault": "is_default",
    "isEnabled": "is_enabled",
    "isOpenPublicIp": "is_open_public_ip",
    "isOutdated": "is_outdated",
    "isPtr": "is_ptr",
    "keyFile": "key_file",
    "keyFingerprint": "key_fingerprint",
    "keyId": "key_id",
    "keyName": "key_name",
    "keyNamePrefix": "key_name_prefix",
    "keyPairName": "key_pair_name",
    "keyUsage": "key_usage",
    "kibanaDomain": "kibana_domain",
    "kibanaPort": "kibana_port",
    "kibanaWhitelists": "kibana_whitelists",
    "kmsEncryptedPassword": "kms_encrypted_password",
    "kmsEncryptionContext": "kms_encryption_context",
    "kmsKeyId": "kms_key_id",
    "kubeConfig": "kube_config",
    "lastModified": "last_modified",
    "lastModifyTime": "last_modify_time",
    "latestImage": "latest_image",
    "launchExpirationTime": "launch_expiration_time",
    "launchTime": "launch_time",
    "lbPort": "lb_port",
    "lbProtocol": "lb_protocol",
    "licenseCode": "license_code",
    "licenseType": "license_type",
    "lifeCycle": "life_cycle",
    "lifecycleRules": "lifecycle_rules",
    "lifecycleTransition": "lifecycle_transition",
    "limitType": "limit_type",
    "listenerForward": "listener_forward",
    "listenerSync": "listener_sync",
    "loadBalancerId": "load_balancer_id",
    "loadbalancerIds": "loadbalancer_ids",
    "localLogRetentionHours": "local_log_retention_hours",
    "localLogRetentionSpace": "local_log_retention_space",
    "localSubnet": "local_subnet",
    "localSubnets": "local_subnets",
    "localTopic": "local_topic",
    "logBackup": "log_backup",
    "logBackupFrequency": "log_backup_frequency",
    "logBackupRetentionPeriod": "log_backup_retention_period",
    "logConfig": "log_config",
    "logEndpoint": "log_endpoint",
    "logRetentionPeriod": "log_retention_period",
    "logSample": "log_sample",
    "loggingEnabled": "logging_enabled",
    "loggingIsenable": "logging_isenable",
    "logtailConfigName": "logtail_config_name",
    "machineGroupName": "machine_group_name",
    "maintainEndTime": "maintain_end_time",
    "maintainStartTime": "maintain_start_time",
    "maintainTime": "maintain_time",
    "masterAutoRenew": "master_auto_renew",
    "masterAutoRenewPeriod": "master_auto_renew_period",
    "masterDbInstanceId": "master_db_instance_id",
    "masterDiskCategory": "master_disk_category",
    "masterDiskSize": "master_disk_size",
    "masterInstanceChargeType": "master_instance_charge_type",
    "masterInstanceType": "master_instance_type",
    "masterInstanceTypes": "master_instance_types",
    "masterNodeSpec": "master_node_spec",
    "masterNodes": "master_nodes",
    "masterPeriod": "master_period",
    "masterPeriodUnit": "master_period_unit",
    "masterPublicIp": "master_public_ip",
    "masterPwd": "master_pwd",
    "masterSlaveServerGroupId": "master_slave_server_group_id",
    "masterZoneId": "master_zone_id",
    "maxBandwidthAbs": "max_bandwidth_abs",
    "maxBandwidthPercent": "max_bandwidth_percent",
    "maxConnections": "max_connections",
    "maxDelayTime": "max_delay_time",
    "maxLoginAttempts": "max_login_attempts",
    "maxPasswordAge": "max_password_age",
    "maxSize": "max_size",
    "maxSplitShardCount": "max_split_shard_count",
    "maxVersion": "max_version",
    "maximumMessageSize": "maximum_message_size",
    "memorySize": "memory_size",
    "messageRetentionPeriod": "message_retention_period",
    "messageType": "message_type",
    "metricName": "metric_name",
    "metricType": "metric_type",
    "mfaBindRequired": "mfa_bind_required",
    "minBandwidthAbs": "min_bandwidth_abs",
    "minBandwidthPercent": "min_bandwidth_percent",
    "minSize": "min_size",
    "minimumPasswordLength": "minimum_password_length",
    "mnsEndpoint": "mns_endpoint",
    "mockServiceConfig": "mock_service_config",
    "mongoLists": "mongo_lists",
    "monitoringPeriod": "monitoring_period",
    "multiAzPolicy": "multi_az_policy",
    "namePrefix": "name_prefix",
    "natGatewayId": "nat_gateway_id",
    "needSlb": "need_slb",
    "netType": "net_type",
    "networkAclId": "network_acl_id",
    "networkInterfaceId": "network_interface_id",
    "networkInterfaces": "network_interfaces",
    "networkType": "network_type",
    "newNatGateway": "new_nat_gateway",
    "nextHop": "next_hop",
    "nexthopId": "nexthop_id",
    "nexthopType": "nexthop_type",
    "nicType": "nic_type",
    "nodeCidrMask": "node_cidr_mask",
    "nodeNumber": "node_number",
    "notificationArn": "notification_arn",
    "notificationMetadata": "notification_metadata",
    "notificationTypes": "notification_types",
    "notifyContentFormat": "notify_content_format",
    "notifyStrategy": "notify_strategy",
    "offeringType": "offering_type",
    "onDemandBaseCapacity": "on_demand_base_capacity",
    "onDemandPercentageAboveBaseCapacity": "on_demand_percentage_above_base_capacity",
    "oppositeAccessPointId": "opposite_access_point_id",
    "oppositeInterfaceId": "opposite_interface_id",
    "oppositeInterfaceOwnerId": "opposite_interface_owner_id",
    "oppositeRegion": "opposite_region",
    "oppositeRouterId": "opposite_router_id",
    "oppositeRouterType": "opposite_router_type",
    "optimizeEnable": "optimize_enable",
    "optionSoftwareLists": "option_software_lists",
    "osType": "os_type",
    "ossBucket": "oss_bucket",
    "ossBucketName": "oss_bucket_name",
    "ossKey": "oss_key",
    "ossKeyPrefix": "oss_key_prefix",
    "ossPrefix": "oss_prefix",
    "otsInstanceName": "ots_instance_name",
    "outputType": "output_type",
    "packageVersion": "package_version",
    "page404Config": "page404_config",
    "pageCompressEnable": "page_compress_enable",
    "paidType": "paid_type",
    "parameterFilterConfig": "parameter_filter_config",
    "partitionNum": "partition_num",
    "passwordInherit": "password_inherit",
    "passwordResetRequired": "password_reset_required",
    "passwordReusePrevention": "password_reuse_prevention",
    "payType": "pay_type",
    "percentSourceType": "percent_source_type",
    "periodUnit": "period_unit",
    "persistenceTimeout": "persistence_timeout",
    "pgpKey": "pgp_key",
    "planCode": "plan_code",
    "podCidr": "pod_cidr",
    "policyName": "policy_name",
    "policyType": "policy_type",
    "pollingWaitSeconds": "polling_wait_seconds",
    "portCount": "port_count",
    "portRange": "port_range",
    "preferredBackupPeriods": "preferred_backup_periods",
    "preferredBackupTime": "preferred_backup_time",
    "pricingCycle": "pricing_cycle",
    "primaryKeys": "primary_keys",
    "privateIp": "private_ip",
    "privateIpAddress": "private_ip_address",
    "privateIps": "private_ips",
    "privateIpsCount": "private_ips_count",
    "privateKey": "private_key",
    "privateWhitelists": "private_whitelists",
    "privateZone": "private_zone",
    "productCode": "product_code",
    "projectName": "project_name",
    "protocolType": "protocol_type",
    "proxyPattern": "proxy_pattern",
    "publicIp": "public_ip",
    "publicKey": "public_key",
    "publicWhitelists": "public_whitelists",
    "publishVpc": "publish_vpc",
    "qosId": "qos_id",
    "ramRoleName": "ram_role_name",
    "ramUsers": "ram_users",
    "rangeEnable": "range_enable",
    "readEnable": "read_enable",
    "recordCount": "record_count",
    "recordId": "record_id",
    "recordSchema": "record_schema",
    "recordType": "record_type",
    "recurrenceEndTime": "recurrence_end_time",
    "recurrenceType": "recurrence_type",
    "recurrenceValue": "recurrence_value",
    "referConfig": "refer_config",
    "refererConfig": "referer_config",
    "regionIds": "region_ids",
    "relatedClusterId": "related_cluster_id",
    "releaseEip": "release_eip",
    "releaseTime": "release_time",
    "remoteSubnets": "remote_subnets",
    "removalPolicies": "removal_policies",
    "renewalStatus": "renewal_status",
    "repeatWeekdays": "repeat_weekdays",
    "replicaSetName": "replica_set_name",
    "replicationFactor": "replication_factor",
    "repoType": "repo_type",
    "requestConfig": "request_config",
    "requestParameters": "request_parameters",
    "requestTimeout": "request_timeout",
    "requireLowercaseCharacters": "require_lowercase_characters",
    "requireNumbers": "require_numbers",
    "requireSymbols": "require_symbols",
    "requireUppercaseCharacters": "require_uppercase_characters",
    "resourceGroupId": "resource_group_id",
    "resourceRecord": "resource_record",
    "retentionDays": "retention_days",
    "retentionPeriod": "retention_period",
    "roleId": "role_id",
    "roleName": "role_name",
    "routeDest": "route_dest",
    "routeTableId": "route_table_id",
    "routerId": "router_id",
    "routerTableId": "router_table_id",
    "routerType": "router_type",
    "rwAccessType": "rw_access_type",
    "sagId": "sag_id",
    "scalingConfigurationName": "scaling_configuration_name",
    "scalingGroupId": "scaling_group_id",
    "scalingGroupName": "scaling_group_name",
    "scalingRuleName": "scaling_rule_name",
    "scalingRuleType": "scaling_rule_type",
    "scheduledAction": "scheduled_action",
    "scheduledTaskName": "scheduled_task_name",
    "secretFile": "secret_file",
    "secretKey": "secret_key",
    "securityEnhancementStrategy": "security_enhancement_strategy",
    "securityGroupId": "security_group_id",
    "securityGroupIds": "security_group_ids",
    "securityGroupType": "security_group_type",
    "securityGroups": "security_groups",
    "securityIpLists": "security_ip_lists",
    "securityIpMode": "security_ip_mode",
    "securityIps": "security_ips",
    "securityToken": "security_token",
    "serverCertificate": "server_certificate",
    "serverCertificateId": "server_certificate_id",
    "serverGroupId": "server_group_id",
    "serverSideEncryption": "server_side_encryption",
    "serverSideEncryptionRule": "server_side_encryption_rule",
    "serverType": "server_type",
    "serviceBandwidth": "service_bandwidth",
    "serviceCidr": "service_cidr",
    "serviceDomain": "service_domain",
    "serviceId": "service_id",
    "serviceType": "service_type",
    "shardCount": "shard_count",
    "shardLists": "shard_lists",
    "sharedCredentialsFile": "shared_credentials_file",
    "silenceTime": "silence_time",
    "skipRegionValidation": "skip_region_validation",
    "slaveZoneId": "slave_zone_id",
    "slbId": "slb_id",
    "slbInternet": "slb_internet",
    "slbInternetEnabled": "slb_internet_enabled",
    "slbIntranet": "slb_intranet",
    "slsProjectArn": "sls_project_arn",
    "slsWriteRoleArn": "sls_write_role_arn",
    "snapshotId": "snapshot_id",
    "snatEntryId": "snat_entry_id",
    "snatIp": "snat_ip",
    "snatTableId": "snat_table_id",
    "snatTableIds": "snat_table_ids",
    "sourceArn": "source_arn",
    "sourceCidr": "source_cidr",
    "sourceCidrIp": "source_cidr_ip",
    "sourceGroupOwnerAccount": "source_group_owner_account",
    "sourceImageId": "source_image_id",
    "sourcePort": "source_port",
    "sourcePortRange": "source_port_range",
    "sourceRegionId": "source_region_id",
    "sourceSecurityGroupId": "source_security_group_id",
    "sourceType": "source_type",
    "sourceVswitchId": "source_vswitch_id",
    "specType": "spec_type",
    "spotInstancePools": "spot_instance_pools",
    "spotInstanceRemedy": "spot_instance_remedy",
    "spotPriceLimit": "spot_price_limit",
    "spotStrategy": "spot_strategy",
    "sqlCollectorConfigValue": "sql_collector_config_value",
    "sqlCollectorStatus": "sql_collector_status",
    "sshEnable": "ssh_enable",
    "sslCertificateId": "ssl_certificate_id",
    "sslConnections": "ssl_connections",
    "sslVpnServerId": "ssl_vpn_server_id",
    "stageName": "stage_name",
    "stageNames": "stage_names",
    "startTime": "start_time",
    "stepAdjustments": "step_adjustments",
    "stickySession": "sticky_session",
    "stickySessionType": "sticky_session_type",
    "storageClass": "storage_class",
    "storageEngine": "storage_engine",
    "storageType": "storage_type",
    "subDomain": "sub_domain",
    "subId": "sub_id",
    "subnetId": "subnet_id",
    "systemDiskCategory": "system_disk_category",
    "systemDiskDescription": "system_disk_description",
    "systemDiskName": "system_disk_name",
    "systemDiskSize": "system_disk_size",
    "systemParameters": "system_parameters",
    "tableName": "table_name",
    "targetValue": "target_value",
    "taskEnabled": "task_enabled",
    "timePoints": "time_points",
    "timeToLive": "time_to_live",
    "tlsCipherPolicy": "tls_cipher_policy",
    "topicName": "topic_name",
    "topicQuota": "topic_quota",
    "triggerId": "trigger_id",
    "triggeredCount": "triggered_count",
    "unhealthyThreshold": "unhealthy_threshold",
    "updateTime": "update_time",
    "useLocalMetadb": "use_local_metadb",
    "userAccessType": "user_access_type",
    "userCa": "user_ca",
    "userClientIp": "user_client_ip",
    "userData": "user_data",
    "userDefinedEmrEcsRole": "user_defined_emr_ecs_role",
    "userMail": "user_mail",
    "userName": "user_name",
    "userNames": "user_names",
    "versionId": "version_id",
    "videoSeekEnable": "video_seek_enable",
    "visibilityTimeout": "visibility_timeout",
    "volumeTags": "volume_tags",
    "vpcAuthMode": "vpc_auth_mode",
    "vpcConfig": "vpc_config",
    "vpcDomain": "vpc_domain",
    "vpcId": "vpc_id",
    "vpcIds": "vpc_ids",
    "vpcName": "vpc_name",
    "vpnGatewayId": "vpn_gateway_id",
    "vserverGroups": "vserver_groups",
    "vswitchId": "vswitch_id",
    "vswitchIds": "vswitch_ids",
    "workerAutoRenew": "worker_auto_renew",
    "workerAutoRenewPeriod": "worker_auto_renew_period",
    "workerDataDiskCategory": "worker_data_disk_category",
    "workerDataDiskSize": "worker_data_disk_size",
    "workerDiskCategory": "worker_disk_category",
    "workerDiskSize": "worker_disk_size",
    "workerInstanceChargeType": "worker_instance_charge_type",
    "workerInstanceType": "worker_instance_type",
    "workerInstanceTypes": "worker_instance_types",
    "workerNodes": "worker_nodes",
    "workerNumber": "worker_number",
    "workerNumbers": "worker_numbers",
    "workerPeriod": "worker_period",
    "workerPeriodUnit": "worker_period_unit",
    "xForwardedFor": "x_forwarded_for",
    "zoneCount": "zone_count",
    "zoneId": "zone_id",
}
